import magicmind.python.runtime as mm
import math
import argparse
import numpy as np
import torch
from torchvision import transforms#, utils
import glob
import os
from skimage import transform, io
from magicmind.python.common.types import get_numpy_dtype_by_datatype
import cv2
from PIL import Image

parser = argparse.ArgumentParser()
parser.add_argument('--device_id', dest = 'device_id', default = 0,
                    type = int, help = 'mlu device id')
parser.add_argument("--magicmind_model", type=str, default="magicmind_model",
                    help="magicmind_model")
parser.add_argument("--img_dir", type=str, help="images directory",
                    required=True)
parser.add_argument("--output_folder", type=str, default="../data/output",
                    help="forder for saving results.")
parser.add_argument('--batch_size', dest = 'batch_size', default = 1,
                    type = int, help = 'batch_size')
parser.add_argument('--save_img', dest = 'save_img', default = False,
                    type = bool, help = 'save result in images')

def normPRED(d):
    ma = torch.max(d)
    mi = torch.min(d)
    dn = (d-mi)/(ma-mi)
    return dn

transform1 = transforms.Compose([
	transforms.ToTensor(), 
	]
)

def eval_mae(y_pred, y):
    return torch.abs(y_pred - y).mean()

beta = math.sqrt(0.3)  # for max F_beta metric
# get precisions and recalls: threshold---divided [0, 1] to num values
def eval_pr(y_pred, y, num):
    prec, recall = torch.zeros(num), torch.zeros(num)
    thlist = torch.linspace(0, 1 - 1e-10, num)
    for i in range(num):
        y_temp = (y_pred >= thlist[i]).float()
        tp = (y_temp * y).sum()
        prec[i], recall[i] = tp / (y_temp.sum() + 1e-20), tp / y.sum()
    return prec, recall

def save_output(image_name,pred,d_dir):

    predict = pred
    predict = predict.squeeze()
    predict_np = predict.cpu().data.numpy()
    im = Image.fromarray(predict_np*255).convert('RGB')
    img_name = image_name.split(os.sep)[-1]
    image = io.imread(image_name)
    imo = im.resize((image.shape[1],image.shape[0]),resample=Image.BILINEAR)
    aaa = img_name.split(".")
    bbb = aaa[0:-1]
    imidx = bbb[0]
    for i in range(1,len(bbb)):
        imidx = imidx + "." + bbb[i]
    imo.save(d_dir+imidx+'.png')

def preprocess(img):
    output_size = (320, 320)
    new_h, new_w = output_size
    image = transform.resize(img,(new_h, new_w),mode='constant')
    tmpImg = np.zeros((image.shape[0], image.shape[1],3))
    image = image/np.max(image)
    if image.shape[2] == 1:
        tmpImg[:,:,0] = (image[:,:,0]-0.485)/0.229
        tmpImg[:,:,1] = (image[:,:,0]-0.485)/0.229
        tmpImg[:,:,2] = (image[:,:,0]-0.485)/0.229
    else:
        tmpImg[:,:,0] = (image[:,:,0]-0.485)/0.229
        tmpImg[:,:,1] = (image[:,:,1]-0.456)/0.224
        tmpImg[:,:,2] = (image[:,:,2]-0.406)/0.225
    tmpImg = tmpImg.transpose((2, 0, 1))
    return tmpImg

def load_processed_image(file_path):
    image = io.imread(file_path)
    tmpImg = preprocess(image)
    return tmpImg

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    model = mm.Model()
    model.deserialize_from_file(args.magicmind_model)

    with mm.System():
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        context = engine.create_i_context()
        queue = dev.create_queue()
        assert queue != None

        inputs = context.create_inputs()
        outputs = []

        img_name_list = glob.glob(args.img_dir + os.sep + '*.jpg')
        label_name_list = glob.glob(args.img_dir + os.sep + '*.png')
        img_name_list.sort()
        label_name_list.sort()
        image_num = len(img_name_list)
        pad_num = args.batch_size - (image_num % args.batch_size)
        if pad_num != args.batch_size:
            for i in range(pad_num):
                img_name_list.append(img_name_list[0])
            image_num += pad_num
        loop_num = 0
        avg_mae = 0.0
        num= 100
        avg_prec, avg_recall = torch.zeros(num), torch.zeros(num)
        while loop_num < image_num:
            images = []
            image_names = []
            for i in range(args.batch_size):
                images.append(load_processed_image(img_name_list[loop_num + i])[np.newaxis, :])
                image_names.append(img_name_list[loop_num+ i])
            _input_data = np.concatenate(tuple(images))
            input_data = np.ascontiguousarray(_input_data.astype(dtype=get_numpy_dtype_by_datatype(mm.DataType.FLOAT32)))
            inputs[0].from_numpy(input_data)
            status =context.enqueue(inputs, outputs, queue)
            queue.sync()

            d1 = torch.from_numpy(outputs[0].asnumpy())
            for i in range(args.batch_size):
                labels = cv2.imread(label_name_list[loop_num + i])
                labels = labels[:,:,0]
                pred = d1[i:i+1,0,:,:]
                pred_ori = normPRED(pred)
                if args.save_img:
                    save_output(image_names[i], pred_ori, args.output_folder)
                pred_s = pred_ori.squeeze()
                pred_np = pred_s.data.numpy()
                predict_np = Image.fromarray(pred_np * 255)
                imo = predict_np.resize((labels.shape[1],labels.shape[0]),resample=Image.BILINEAR)
                imo = transform1(imo)
                labels = torch.tensor(labels)
                labels = labels.unsqueeze(0)
                imo = imo.permute(1,2,0)
                labels = labels.permute(1,2,0)
                mae = eval_mae(imo, labels)
                prec, recall = eval_pr(imo, labels, num)
                avg_mae += mae
                avg_prec, avg_recall = avg_prec + prec, avg_recall + recall

            loop_num += args.batch_size

        avg_mae, avg_prec, avg_recall = avg_mae / image_num, avg_prec / image_num, avg_recall / image_num
        score = (1 + beta ** 2) * avg_prec * avg_recall / (beta ** 2 * avg_prec + avg_recall)
        score[score != score] = 0  # delete the nan
        print('average mae: %.4f, max fmeasure: %.4f' % (avg_mae, score.max()))
        f = open(args.output_folder+os.sep+"result.txt", "w")
        f.write('average mae: %.4f\n' %(avg_mae))
        f.write('max fmeasure: %.4f\n' %(score.max()))
        f.close()





#!/usr/bin/python
# encoding: utf-8

import torch
import torch.nn as nn
from torch.autograd import Variable
import collections


class strLabelConverter(object):
    """Convert between str and label.

    NOTE:
        Insert `blank` to the alphabet for CTC.

    Args:
        alphabet (str): set of the possible characters.
        ignore_case (bool, default=True): whether or not to ignore all of the case.
    """

    def __init__(self, alphabet, ignore_case=True):
        self._ignore_case = ignore_case
        if self._ignore_case:
            alphabet = alphabet.lower()
        self.alphabet = alphabet + "-"  # for `-1` index

        self.dict = {}
        for i, char in enumerate(alphabet):
            # NOTE: 0 is reserved for 'blank' required by wrap_ctc
            self.dict[char] = i + 1

    def encode(self, text):
        """Support batch or single str.

        Args:
            text (str or list of str): texts to convert.

        Returns:
            torch.IntTensor [length_0 + length_1 + ... length_{n - 1}]: encoded texts.
            torch.IntTensor [n]: length of each text.
        """
        if isinstance(text, str):

            text = [
                self.dict[char.lower() if self._ignore_case else char] for char in text
            ]
            length = [len(text)]

        elif isinstance(text, collections.Iterable):
            length = [len(s) for s in text]
            text = "".join(text)
            text, _ = self.encode(text)

        return (torch.IntTensor(text), torch.IntTensor(length))

    def decode(self, t, length, raw=False):
        """Decode encoded texts back into strs.

        Args:
            torch.IntTensor [length_0 + length_1 + ... length_{n - 1}]: encoded texts.
            torch.IntTensor [n]: length of each text.

        Raises:
            AssertionError: when the texts and its length does not match.
        Returns:
            text (str or list of str): texts to convert.
        """
        if length.numel() == 1:
            length = length[0]
            assert (
                t.numel() == length
            ), "text with length: {} does not match declared length: {}".format(
                t.numel(), length
            )
            if raw:
                return "".join([self.alphabet[i - 1] for i in t])
            else:
                char_list = []
                for i in range(length):
                    if t[i] != 0 and (not (i > 0 and t[i - 1] == t[i])):
                        char_list.append(self.alphabet[t[i] - 1])
                return "".join(char_list)
        else:
            # batch mode
            assert (
                t.numel() == length.sum()
            ), "texts with length: {} does not match declared length: {}".format(
                t.numel(), length.sum()
            )
            texts = []
            index = 0
            for i in range(length.numel()):
                l = length[i]
                texts.append(
                    self.decode(t[index:index + l], torch.IntTensor([l]), raw=raw)
                )
                index += l
            return texts
class averager(object):
    """Compute average for `torch.Variable` and `torch.Tensor`."""

    def __init__(self):
        self.reset()

    def add(self, v):
        if isinstance(v, Variable):
            count = v.data.numel()
            v = v.data.sum()
        elif isinstance(v, torch.Tensor):
            count = v.numel()
            v = v.sum()
        elif isinstance(v, float):
            count = 1

        self.n_count += count
        self.sum += v

    def reset(self):
        self.n_count = 0
        self.sum = 0

    def val(self):
        res = 0
        if self.n_count != 0:
            res = self.sum / self.n_count
        return res


def oneHot(v, v_length, nc):
    batchSize = v_length.size(0)
    maxLength = v_length.max()
    v_onehot = torch.FloatTensor(batchSize, maxLength, nc).fill_(0)
    acc = 0
    for i in range(batchSize):
        length = v_length[i]
        label = v[acc:acc + length].view(-1, 1).long()
        v_onehot[i, :length].scatter_(1, label, 1.0)
        acc += length
    return v_onehot


def loadData(v, data):
    with torch.no_grad():
        v.resize_(data.size()).copy_(data)


def prettyPrint(v):
    print("Size {0}, Type: {1}".format(str(v.size()), v.data.type()))
    print(
        "| Max: %f | Min: %f | Mean: %f" 
        % (v.max().data[0], v.min().data[0],v.mean().data[0])
    )


def assureRatio(img):
    """Ensure imgH <= imgW."""
    b, c, h, w = img.size()
    if h > w:
        main = nn.UpsamplingBilinear2d(size=(h, h), scale_factor=None)
        img = main(img)
    return img




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from collections import namedtuple
import numpy as np
from shapely.geometry import Polygon
import json
import argparse
"""
reference from :
https://github.com/MhLiao/DB/blob/3c32b808d4412680310d3d28eeb6a2d5bf1566c5/concern/icdar2015_eval/detection/iou.py#L8
"""


class DetectionIoUEvaluator(object):
    def __init__(self, iou_constraint=0.5, area_precision_constraint=0.5):
        self.iou_constraint = iou_constraint
        self.area_precision_constraint = area_precision_constraint

    def evaluate_image(self, gt, pred):
        def get_union(pD, pG):
            return Polygon(pD).union(Polygon(pG)).area

        def get_intersection_over_union(pD, pG):
            return get_intersection(pD, pG) / get_union(pD, pG)

        def get_intersection(pD, pG):
            return Polygon(pD).intersection(Polygon(pG)).area

        def compute_ap(confList, matchList, numGtCare):
            correct = 0
            AP = 0
            if len(confList) > 0:
                confList = np.array(confList)
                matchList = np.array(matchList)
                sorted_ind = np.argsort(-confList)
                confList = confList[sorted_ind]
                matchList = matchList[sorted_ind]
                for n in range(len(confList)):
                    match = matchList[n]
                    if match:
                        correct += 1
                        AP += float(correct) / (n + 1)

                if numGtCare > 0:
                    AP /= numGtCare

            return AP

        perSampleMetrics = {}

        matchedSum = 0

        Rectangle = namedtuple('Rectangle', 'xmin ymin xmax ymax')

        numGlobalCareGt = 0
        numGlobalCareDet = 0

        arrGlobalConfidences = []
        arrGlobalMatches = []

        recall = 0
        precision = 0
        hmean = 0

        detMatched = 0

        iouMat = np.empty([1, 1])

        gtPols = []
        detPols = []

        gtPolPoints = []
        detPolPoints = []

        # Array of Ground Truth Polygons' keys marked as don't Care
        gtDontCarePolsNum = []
        # Array of Detected Polygons' matched with a don't Care GT
        detDontCarePolsNum = []

        pairs = []
        detMatchedNums = []

        arrSampleConfidences = []
        arrSampleMatch = []

        evaluationLog = ""

        for n in range(len(gt)):
            points = gt[n]['points']
            if gt[n]['transcription'] == '###':
                dontCare = True
            else:
                dontCare = False
            if not Polygon(points).is_valid or not Polygon(points).is_simple:
                continue

            gtPol = points
            gtPols.append(gtPol)
            gtPolPoints.append(points)
            if dontCare:
                gtDontCarePolsNum.append(len(gtPols) - 1)

        evaluationLog += "GT polygons: " + str(len(gtPols)) + (
            " (" + str(len(gtDontCarePolsNum)) + " don't care)\n"
            if len(gtDontCarePolsNum) > 0 else "\n")

        for n in range(len(pred)):
            points = pred[n]['points']
            detPol = points
            detPols.append(detPol)
            detPolPoints.append(points)
            if len(gtDontCarePolsNum) > 0:
                for dontCarePol in gtDontCarePolsNum:
                    dontCarePol = gtPols[dontCarePol]
                    intersected_area = get_intersection(dontCarePol, detPol)
                    pdDimensions = Polygon(detPol).area
                    precision = 0 if pdDimensions == 0 else intersected_area / pdDimensions
                    if (precision > self.area_precision_constraint):
                        detDontCarePolsNum.append(len(detPols) - 1)
                        break

        evaluationLog += "DET polygons: " + str(len(detPols)) + (
            " (" + str(len(detDontCarePolsNum)) + " don't care)\n"
            if len(detDontCarePolsNum) > 0 else "\n")

        if len(gtPols) > 0 and len(detPols) > 0:
            # Calculate IoU and precision matrixs
            outputShape = [len(gtPols), len(detPols)]
            iouMat = np.empty(outputShape)
            gtRectMat = np.zeros(len(gtPols), np.int8)
            detRectMat = np.zeros(len(detPols), np.int8)
            for gtNum in range(len(gtPols)):
                for detNum in range(len(detPols)):
                    pG = gtPols[gtNum]
                    pD = detPols[detNum]
                    iouMat[gtNum, detNum] = get_intersection_over_union(pD, pG)

            for gtNum in range(len(gtPols)):
                for detNum in range(len(detPols)):
                    if gtRectMat[gtNum] == 0 and detRectMat[
                            detNum] == 0 and gtNum not in gtDontCarePolsNum and detNum not in detDontCarePolsNum:
                        if iouMat[gtNum, detNum] > self.iou_constraint:
                            gtRectMat[gtNum] = 1
                            detRectMat[detNum] = 1
                            detMatched += 1
                            pairs.append({'gt': gtNum, 'det': detNum})
                            detMatchedNums.append(detNum)
                            evaluationLog += "Match GT #" + \
                                             str(gtNum) + " with Det #" + str(detNum) + "\n"

        numGtCare = (len(gtPols) - len(gtDontCarePolsNum))
        numDetCare = (len(detPols) - len(detDontCarePolsNum))
        if numGtCare == 0:
            recall = float(1)
            precision = float(0) if numDetCare > 0 else float(1)
        else:
            recall = float(detMatched) / numGtCare
            precision = 0 if numDetCare == 0 else float(detMatched) / numDetCare

        hmean = 0 if (precision + recall) == 0 else 2.0 * \
                                                    precision * recall / (precision + recall)

        matchedSum += detMatched
        numGlobalCareGt += numGtCare
        numGlobalCareDet += numDetCare

        perSampleMetrics = {
            'precision': precision,
            'recall': recall,
            'hmean': hmean,
            'pairs': pairs,
            'iouMat': [] if len(detPols) > 100 else iouMat.tolist(),
            'gtPolPoints': gtPolPoints,
            'detPolPoints': detPolPoints,
            'gtCare': numGtCare,
            'detCare': numDetCare,
            'gtDontCare': gtDontCarePolsNum,
            'detDontCare': detDontCarePolsNum,
            'detMatched': detMatched,
            'evaluationLog': evaluationLog
        }

        return perSampleMetrics

    def combine_results(self, results):
        numGlobalCareGt = 0
        numGlobalCareDet = 0
        matchedSum = 0
        for result in results:
            numGlobalCareGt += result['gtCare']
            numGlobalCareDet += result['detCare']
            matchedSum += result['detMatched']
        methodRecall = 0 if numGlobalCareGt == 0 else float(
            matchedSum) / numGlobalCareGt
        methodPrecision = 0 if numGlobalCareDet == 0 else float(
            matchedSum) / numGlobalCareDet
        methodHmean = 0 if methodRecall + methodPrecision == 0 else 2 * \
                                                                    methodRecall * methodPrecision / (
                                                                            methodRecall + methodPrecision)        
        methodMetrics = {
            'precision': methodPrecision,
            'recall': methodRecall,
            'hmean': methodHmean
        }
        return methodMetrics

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--label_file', type=str, default='icdar2015.json', help='icdar2015 val json label')
    parser.add_argument('--result_dir', type=str, default='./data/psenet_pred.json', help='prediction json file')
    args = parser.parse_args()
    evaluator = DetectionIoUEvaluator()
    label_file = args.label_file
    pred_file = args.result_dir

    with open(pred_file, 'r') as fp:
        preds_ = json.load(fp)
    with open(label_file, 'r') as fl:
        labels = json.load(fl)
    pre = 0.
    recall =0.
    hmean = 0.
    results = []
    for key in preds_.keys():
        gts = [labels[key]]
        preds = [preds_[key]]
        for gt, pred in zip(gts, preds):
            try:
                results.append(evaluator.evaluate_image(gt, pred))
            except:
                print(key)
    metrics = evaluator.combine_results(results)
    pre = metrics['precision'] * 100
    recall = metrics['recall'] * 100 
    hmean = metrics['hmean'] * 100
    print("=" * 43)
    print("precision:{:.2f}%".format(pre), "recall:{:.2f}%".format(recall), "hmean:{:.2f}%".format(hmean))
    print("=" * 43)




import os
from PIL import Image
import numpy as np
import magicmind.python.runtime as mm
import argparse
import sys
import clip
import torch
from torchvision.datasets import CIFAR100
from torch.utils.data import DataLoader
from tqdm import tqdm
sys.path.append("../../../")
from utils import Record
from mm_runner import MMRunner
from logger import Logger
log = Logger()
IMAGE_HEIGHT = 224
IMAGE_WIDTH = 224


parser = argparse.ArgumentParser()
parser.add_argument(
  "--device_id", 
  dest="device_id",  
  type=int, 
  default=0, 
  help="device_id"
)
parser.add_argument(
  "--magicmind_model", 
  dest="magicmind_model", 
  type=str, 
  default="../data/models/clip_onnx", 
  help="save mm model to this path"
)
parser.add_argument(
  "--image_dir", 
  dest="image_dir",  
  type=str, 
  default="/path/to/datasets/cifar100/",
   help="cifar100 val datasets"
)
parser.add_argument(
  "--result_file", 
  dest="result_file", 
  type=str, 
  default="../data/images/output/infer_result.txt", 
  help="result_file"
)
parser.add_argument(
  "--result_label_file", 
  dest="result_label_file",  
  type=str, 
  default="../data/images/output/eval_labels.txt", 
  help="result_label_file"
)
parser.add_argument(
  "--result_top1_file", 
  dest="result_top1_file",  
  type=str, 
  default="../data/images/output/eval_result_1.txt", 
  help="result_top1_file"
)
parser.add_argument(
  "--result_top5_file", 
  dest="result_top5_file",  
  type=str, 
  default="../data/images/output/eval_result_5.txt", 
  help="result_top5_file"
)
parser.add_argument(
  "--batch_size", 
  dest="batch_size",  
  type=int, 
  default=1, 
  help="batch_size"
)
parser.add_argument(
    "--image_num", 
    dest="image_num", 
    type=int, 
    default=1000, 
    help="image number"
)


if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    record = Record(args.result_file)
    result_label = Record(args.result_label_file)
    result_top1 = Record(args.result_top1_file)
    result_top5 = Record(args.result_top5_file)
    clipmodel, preprocess = clip.load('ViT-B/32', "cpu")
    root = os.path.expanduser(args.image_dir)
    test = CIFAR100(root, download=True, train=False, transform=preprocess)
    text_inputs = torch.cat([clip.tokenize(f"a photo of a {c}") for c in test.classes]).to("cpu")
    count = 0
    log.info("Start run ...")
    batch_size = args.batch_size
    image_num = args.image_num
    rem_img_num = image_num % batch_size
    img_idx = 0
    batch_counter = 0
    inputs=[]
    total_tmp = int(image_num/batch_size)
    if total_tmp*batch_size < image_num:
        total_tmp += 1

    imgs = np.empty([batch_size, 3, IMAGE_HEIGHT, IMAGE_WIDTH])
    val_loader = DataLoader(test,batch_size=args.batch_size,drop_last=False)
    for images, labels in tqdm(val_loader,total=total_tmp,ncols = 80):
      infer_batch_size = (
            batch_size if img_idx < (image_num - rem_img_num) else rem_img_num
      )
      if img_idx >= image_num:
            break
      for index in range (infer_batch_size):
            image = images[index]
            imgs[batch_counter % infer_batch_size, :, :, :] = image
            batch_counter += 1
            img_idx += 1
      if batch_counter % infer_batch_size == 0:
            inputs.append(imgs)
            inputs.append(text_inputs.numpy())
            outputs = model(inputs)
            logits_per_image = torch.from_numpy(outputs[0])
            logits_per_text = torch.from_numpy(outputs[1])
            probs = logits_per_image.softmax(dim=-1).cpu()
            vals, topk1 = torch.topk(probs,1)
            vals, topk5 = torch.topk(probs,5)
            for pred_idx in range(infer_batch_size):
              record.write("top5 result:", False)
              result_label.write("[%d]: %d"%(count, labels[pred_idx]), False)
              result_top1.write("[%d]: %d"%(count, topk1[pred_idx][0]), False)
              for j in range(5):
                idx = topk5[pred_idx][j]
                name = test.classes[idx]
                record.write("%d [%s]"%(j, name), False)
                result_top5.write("[%d]: %d"%(count, idx), False)
              count += 1



import torch
import os
import transformers
from transformers import BertModel
import argparse
import warnings
from transformers import AutoTokenizer, BertForSequenceClassification
warnings.filterwarnings("ignore")

parser = argparse.ArgumentParser()
parser.add_argument("--model_path", "--model_path", type=str, default="../data/models/chinese-roberta-wwm-ext", help="model_path")
parser.add_argument("--onnx_model", "--onnx_model", type=str, default="../data/models/roberta.onnx", help="robert onnx")
parser.add_argument("--batch_size", "--batch_size", type=int, default=16, help="batch_size")
parser.add_argument("--max_seq_length", "--max_seq_length", type=int, default=128, help="max_seq_length")

if __name__ == "__main__":
    args = parser.parse_args()
    pt_model = BertForSequenceClassification.from_pretrained(args.model_path, from_tf = False, config = os.path.join(args.model_path, "config.json"), num_labels=2, cache_dir = None)
    pt_model.eval()

    tokens = torch.randint(0, 1, (args.batch_size, args.max_seq_length), dtype=torch.long)
    segments = torch.randint(0, 1, (args.batch_size, args.max_seq_length), dtype=torch.long)
    mask = torch.randint(0, 1, (args.batch_size, args.max_seq_length), dtype=torch.long)

    # 导出模型到 ONNX，指定可变维度的名称
    input_names = ['input_ids', 'attention_mask', 'token_type_ids']
    output_names = ['output']
    dynamic_axes = {'input_ids': {0: 'batch_size', 1: 'seq_length'},
                    'attention_mask': {0: 'batch_size', 1: 'seq_length'},
                    'token_type_ids': {0: 'batch_size', 1: 'seq_length'},
                    'output': {0: 'batch_size'}}

    torch.onnx.export(pt_model, (tokens, segments, mask), args.onnx_model, verbose=True, opset_version=11, input_names=input_names, output_names=output_names, dynamic_axes=dynamic_axes)
    print("successfully save onnx")



import os
import sys
import numpy as np

from PIL import Image
import argparse
import logging
from tqdm import tqdm

import magicmind.python.runtime as mm
from torchvision import transforms

from utils import Record
from mm_runner import MMRunner
from logger import Logger

log = Logger()


def imagenet_dataset(val_txt, image_file_path, count=-1):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    log.info("{} pictures will be read.".format(len(lines)))
    if len(lines) < count:
        log.info("infer pictures less than {}".format(count))
        return
    current_count = 0
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        img = Image.open(image_path)
        if img.mode != "RGB":
            img = img.convert("RGB")
        yield img, label.strip()
        current_count += 1
        if current_count >= count and count != -1:
            break


def preprocess(input_image, transpose):
    resize_h, resize_w = (232, 232)
    crop_h, crop_w = (224, 224)
    preprocess = transforms.Compose(
        [
            transforms.Resize(resize_h, interpolation=Image.BICUBIC),
            transforms.CenterCrop(crop_h),
        ]
    )
    input_tensor = preprocess(input_image)
    input_numpy = np.array(input_tensor)
    return input_numpy


def load_name(imagenet_label_path):
    name_map = {}
    with open(imagenet_label_path, "r") as f:
        lines = f.readlines()
    for line in lines:
        idx = line.split(" ")[0]
        name = " ".join(line.split(" ")[1:])
        name_map[int(idx)] = name.strip()
    return name_map


parser = argparse.ArgumentParser()
parser.add_argument("--device_id", "--device_id", type=int, default=0, help="device_id")
parser.add_argument(
    "--magicmind_model",
    "--magicmind_model",
    type=str,
    default="../data/models/swin_onnx_model",
    help="save mm model to this path",
)
parser.add_argument(
    "--image_dir",
    "--image_dir",
    type=str,
    default="/path/to/modelzoo/datasets/imageNet2012/",
    help="imagenet val datasets",
)
parser.add_argument(
    "--image_num", "--image_num", type=int, default=10, help="image number"
)
parser.add_argument(
    "--name_file",
    "--name_file",
    type=str,
    default="datasets/imagenet/name.txt",
    help="imagenet name txt",
)
parser.add_argument(
    "--label_file",
    "--label_file",
    type=str,
    default="/path/to/modelzoo/datasets/imageNet2012/labels.txt",
    help="imagenet val label txt",
)
parser.add_argument(
    "--result_file",
    "--result_file",
    type=str,
    default="../data/images/output/infer_result.txt",
    help="result_file",
)
parser.add_argument(
    "--result_label_file",
    "--result_label_file",
    type=str,
    default="../data/images/output/eval_labels.txt",
    help="result_label_file",
)
parser.add_argument(
    "--result_top1_file",
    "--result_top1_file",
    type=str,
    default="../data/images/output/eval_result_1.txt",
    help="result_top1_file",
)
parser.add_argument(
    "--result_top5_file",
    "--result_top5_file",
    type=str,
    default="../data/images/output/eval_result_5.txt",
    help="result_top5_file",
)
parser.add_argument(
    "--batch_size", "--batch_size", type=int, default=1, help="batch_size"
)

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        log.info("please generate magicmind model first!!!")
        exit()

    # model 定义
    model = MMRunner(mm_file=args.magicmind_model, device_id=args.device_id)

    batch_size = args.batch_size
    image_num = args.image_num

    name_map = load_name(args.name_file)
    record = Record(args.result_file)
    result_label = Record(args.result_label_file)
    result_top1 = Record(args.result_top1_file)
    result_top5 = Record(args.result_top5_file)

    dataset = imagenet_dataset(
        val_txt=args.label_file, image_file_path=args.image_dir, count=image_num
    )
    count = 0
    log.info("Start run ...")

    rem_img_num = image_num % batch_size
    img_idx = 0
    batch_counter = 0
    count = 0
    imgsz = 224
    imgs = np.empty([batch_size, imgsz, imgsz, 3], dtype=np.float)
    labels = [None for i in range(batch_size)]
    for img, label in tqdm(dataset, total=image_num):
        # calc infer_batch_size
        infer_batch_size = (
            batch_size if img_idx < (image_num - rem_img_num) else rem_img_num
        )

        # pre-process
        imgs[batch_counter % infer_batch_size, :, :, :] = preprocess(
            img, transpose=True
        )
        labels[batch_counter % infer_batch_size] = label

        batch_counter += 1
        img_idx += 1

        if batch_counter % infer_batch_size == 0:
            batch_counter = 0
            inputs = [imgs]

            # inference
            outputs = model(inputs)

            # post-process
            for pred_idx in range(infer_batch_size):
                index = outputs[0][pred_idx].argsort()[::-1]
                record.write("top5 result:", False)
                result_label.write("[%d]: %d" % (count, int(labels[pred_idx])), False)
                result_top1.write("[%d]: %d" % (count, index[0]), False)
                for i in range(5):
                    idx = index[i]
                    name = name_map[idx]
                    record.write("%d [%s]" % (i, name), False)
                    result_top5.write("[%d]: %d" % (count, idx), False)
                count += 1



import numpy as np
import magicmind.python.runtime as mm
from magicmind.python.common.types import get_datatype_by_numpy
from typing import List

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, sample_data: List):
        super().__init__()
        self.sample_data = sample_data
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.sample_data))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))
        
    def get_shape(self):
        return mm.Dims(self.cur_sample_.shape)

    def get_data_type(self):
        return mm.DataType.FLOAT32
    
    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = self.sample_data[i]
            imgs.append(img[np.newaxis,:])
        # batch and normalize
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))
    
    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()
    
    def new_video(self, path):
        self.frame = 0
        self.cap = cv2.VideoCapture(path)
        self.nframes = int(self.cap.get(cv2.CAP_PROP_FRAME_COUNT))

    def __len__(self):
        return self.nf  # number of files


from transformers import AutoModelForSequenceClassification
import torch
from datasets import load_dataset
import sys
import os

if __name__=="__main__":
    MODEL_NAME = 'textattack/bert-base-uncased-MRPC'
    PRJ_ROOT_PATH = sys.argv[1]
    BATCH_SIZE = 1
    MAX_SEQ_LENGTH = 128
    
    PYTORCH_MODEL_PATH = os.path.join(PRJ_ROOT_PATH,'data/models/traced.pt')
    if not os.path.exists(PYTORCH_MODEL_PATH):
        save_dir = os.path.join(PRJ_ROOT_PATH,'data/models')
        if not os.path.exists(save_dir):
            os.mkdir(save_dir)
        torch_model = AutoModelForSequenceClassification.from_pretrained(MODEL_NAME,cache_dir=save_dir)
        torch_model.config.return_dict = False 
        
        input_ids = torch.randint(0, 1, (BATCH_SIZE, MAX_SEQ_LENGTH))
        attention_mask = torch.randint(0, 1, (BATCH_SIZE, MAX_SEQ_LENGTH))
        token_type_ids = torch.randint(0, 1, (BATCH_SIZE, MAX_SEQ_LENGTH))
        torch.jit.save(torch.jit.trace(torch_model, (input_ids, attention_mask, token_type_ids)), PYTORCH_MODEL_PATH)
        print("Export torch model to pt model sucess!")
    

    
    


import shutil
import magicmind.python.runtime as mm
from batchgenerators.utilities.file_and_folder_operations import *
from batchgenerators.augmentations.utils import pad_nd_image
import argparse
from typing import Union, Tuple
import numpy as np
from collections import OrderedDict
import torch
import torch.nn as nn
import torch.nn.functional as F
from nnunet.preprocessing.preprocessing import PreprocessorFor2D
#from nnunet.preprocessing.segmentation_export import save_segmentation_nifti_from_softmax
from nnunet.inference.segmentation_export import save_segmentation_nifti_from_softmax
from magicmind.python.runtime import ModelKind
#from nnunet.utils import NNUNetCalibData, check_input_folder_and_return_caseIDs, compute_steps_for_sliding_window
#from calibrator import TorchCalibrator
#from model import MM_Model
from nnunet.evaluation.evaluator import evaluate_folder
from copy import deepcopy

def check_input_folder_and_return_caseIDs(input_folder, expected_num_modalities):
    print("This model expects %d input modalities for each image" % expected_num_modalities)
    files = subfiles(input_folder, suffix=".nii.gz", join=False, sort=True)

    maybe_case_ids = np.unique([i[:-12] for i in files])

    remaining = deepcopy(files)
    missing = []

    assert len(files) > 0, "input folder did not contain any images (expected to find .nii.gz file endings)"

    # now check if all required files are present and that no unexpected files are remaining
    for c in maybe_case_ids:
        for n in range(expected_num_modalities):
            expected_output_file = c + "_%04.0d.nii.gz" % n
            if not isfile(join(input_folder, expected_output_file)):
                missing.append(expected_output_file)
            else:
                remaining.remove(expected_output_file)

    print("Found %d unique case ids, here are some examples:" % len(maybe_case_ids),
          np.random.choice(maybe_case_ids, min(len(maybe_case_ids), 10)))
    print("If they don't look right, make sure to double check your filenames. They must end with _0000.nii.gz etc")

    if len(remaining) > 0:
        print("found %d unexpected remaining files in the folder. Here are some examples:" % len(remaining),
              np.random.choice(remaining, min(len(remaining), 10)))

    if len(missing) > 0:
        print("Some files are missing:")
        print(missing)
        raise RuntimeError("missing files in input_folder")

    return maybe_case_ids


def compute_steps_for_sliding_window(patch_size: Tuple[int, ...], image_size: Tuple[int, ...], step_size: float) -> List[List[int]]:
        import numpy as np
        assert [i >= j for i, j in zip(image_size, patch_size)], "image size must be as large or larger than patch_size"
        assert 0 < step_size <= 1, 'step_size must be larger than 0 and smaller or equal to 1'

        # our step width is patch_size*step_size at most, but can be narrower. For example if we have image size of
        # 110, patch size of 32 and step_size of 0.5, then we want to make 4 steps starting at coordinate 0, 27, 55, 78
        target_step_sizes_in_voxels = [i * step_size for i in patch_size]

        num_steps = [int(np.ceil((i - k) / j)) + 1 for i, j, k in zip(image_size, target_step_sizes_in_voxels, patch_size)]

        steps = []
        for dim in range(len(patch_size)):
            # the highest step value for this dimension is
            max_step_value = image_size[dim] - patch_size[dim]
            if num_steps[dim] > 1:
                actual_step_size = max_step_value / (num_steps[dim] - 1)
            else:
                actual_step_size = 99999999999  # does not matter because there is only one step at 0

            steps_here = [int(np.round(actual_step_size * i)) for i in range(num_steps[dim])]

            steps.append(steps_here)

        return steps


def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")


parser = argparse.ArgumentParser()
parser.add_argument("--data_folder",
                    type=str,
                    help="nnUNet_raw_data_base/nnUNet_raw_data/Task002_Heart/imagesTr",
                    required=True)
parser.add_argument(
    "--model_path",
    type=str,
    help="nnUNet_trained_models/nnUNet/2d/Task002_Heart/nnUNetTrainerV2__nnUNetPlansv2.1",
    required=True)
parser.add_argument("--output_folder",
                    type=str,
                    default="output_f",
                    help="forder for saving results.")
parser.add_argument("--softmax_output_folder",
                    type=str,
                    default="/workspace/softmax_output",
                    help="forder for saving results.")
parser.add_argument("--ref_folder",
                    type=str,
                    help="nnUNet_raw_data_base/nnUNet_raw_data/Task02_Heart/labelsTr",
                    required=False)
args = parser.parse_args()


info = load_pickle(join(args.model_path + "/fold_0/", "%s.model.pkl" % "model_final_checkpoint"))
maybe_mkdir_p(args.output_folder)
shutil.copy(join(args.model_path, 'plans.pkl'), args.output_folder)

assert isfile(join(args.model_path,
                   "plans.pkl")), "Folder with saved model weights must contain a plans.pkl file"
expected_num_modalities = load_pickle(join(args.model_path, "plans.pkl"))['num_modalities']

# check input folder integrity
case_ids = check_input_folder_and_return_caseIDs(args.data_folder, expected_num_modalities)

output_files = [join(args.output_folder, i + ".nii.gz") for i in case_ids]
all_files = subfiles(args.data_folder, suffix=".nii.gz", join=False, sort=True)
list_of_lists = [[
    join(args.data_folder, i) for i in all_files
    if i[:len(j)].startswith(j) and len(i) == (len(j) + 12)
] for j in case_ids]

plans = info['plans']
normalization_schemes = plans['normalization_schemes']
use_mask_for_norm = plans['use_mask_for_norm']
transpose_forward = plans['transpose_forward']
intensity_properties = plans['dataset_properties']['intensityproperties']
transpose_backward = plans['transpose_backward']

init = info['init']
_, _, _, _, _, stage, _, _, _ = init
stage_plans = plans['plans_per_stage'][stage]
batch_size = stage_plans['batch_size']
net_pool_per_axis = stage_plans['num_pool_per_axis']
patch_size = np.array(stage_plans['patch_size']).astype(int)
do_dummy_2D_aug = stage_plans['do_dummy_2D_data_aug']
target_spacing = plans['plans_per_stage'][stage]['current_spacing']
pad_border_mode = 'constant'
pad_kwargs = {'constant_values': 0}
step_size = 0.5
num_classes = 2
mirror_idx = 4
softmax_helper = lambda x: F.softmax(x, 1)
inference_apply_nonlin = softmax_helper
num_results = 4
mirror_axes = (0, 1)
regions_class_order = None

if 'segmentation_export_params' in plans.keys():
    force_separate_z = plans['segmentation_export_params']['force_separate_z']
    interpolation_order = plans['segmentation_export_params']['interpolation_order']
    interpolation_order_z = plans['segmentation_export_params']['interpolation_order_z']
else:
    force_separate_z = None
    interpolation_order = 1
    interpolation_order_z = 0

# TODO: extend for 3d
preprocessor = PreprocessorFor2D(normalization_schemes, use_mask_for_norm, transpose_forward,
                                intensity_properties)


with torch.no_grad():
    for i, file_list in enumerate(list_of_lists):
        output_filename = output_files[i]
        #d, s, dct = preprocessor.preprocess_test_case(file_list, target_spacing)
        name = file_list[0][:-7]
        dct =torch.load(name + "_dct.pt")
        print(name)
        base_name = os.path.basename(name)
        data_shape = np.fromfile(name + "_data_shape_info", dtype=np.int32)
        softmax_pred = []
        for index in range(data_shape[1]):
            output = np.fromfile(args.softmax_output_folder + "/" + base_name + "_data_output_" + str(index), dtype=np.float32)
            output_shape = [data_shape[2], data_shape[3],2]
            output = output.reshape(output_shape)
            predicted_patch = output.transpose((2,0,1))
            softmax_pred.append(predicted_patch[None])
        softmax = np.vstack(softmax_pred).transpose((1, 0, 2, 3))
        npz_file = None
        region_class_order = None
        bytes_per_voxel = 4
        if transpose_forward is not None:
            softmax = softmax.transpose([0] + [i + 1 for i in transpose_backward])
        save_segmentation_nifti_from_softmax(softmax, output_filename, dct, interpolation_order, \
            region_class_order, None, None, npz_file, None, force_separate_z, interpolation_order_z)

evaluate_folder(folder_with_gts=args.ref_folder,
                folder_with_predictions=args.output_folder,
                labels=[1])
with open(args.output_folder + '/summary.json', 'r') as f:
    import json
    jstr = json.load(f)
    print('mean acc:', jstr['results']['mean'])
    print('see', args.output_folder + '/summary.json', 'for detail.')




import abc
import numpy as np
class Metric(metaclass=abc.ABCMeta):
    def __init__(self):
        pass

    @abc.abstractmethod
    def reset(self):
        raise NotImplementedError(
            "function 'reset' not implemented in {}.".format(
                self.__class__.__name__
            )
        )

    @abc.abstractmethod
    def update(self, *args):
        raise NotImplementedError(
            "function 'update' not implemented in {}.".format(
                self.__class__.__name__
            )
        )

    @abc.abstractmethod
    def accumulate(self):
        raise NotImplementedError(
            "function 'accumulate' not implemented in {}.".format(
                self.__class__.__name__
            )
        )

    @abc.abstractmethod
    def name(self):
        raise NotImplementedError(
            "function 'name' not implemented in {}.".format(
                self.__class__.__name__
            )
        )

    def compute(self, *args):
        return args




class Auc(Metric):
    def __init__(
        self, curve='ROC', num_thresholds=4095, name='auc', *args, **kwargs
    ):
        super(Auc, self).__init__(*args, **kwargs)
        self._curve = curve
        self._num_thresholds = num_thresholds

        _num_pred_buckets = num_thresholds + 1
        self._stat_pos = np.zeros(_num_pred_buckets)
        self._stat_neg = np.zeros(_num_pred_buckets)
        self._name = name

    def update(self, preds, labels):
        for i, lbl in enumerate(labels):
            value = preds[i, 1]
            bin_idx = int(value * self._num_thresholds)
            assert bin_idx <= self._num_thresholds
            if lbl:
                self._stat_pos[bin_idx] += 1.0
            else:
                self._stat_neg[bin_idx] += 1.0

    @staticmethod
    def trapezoid_area(x1, x2, y1, y2):
        return abs(x1 - x2) * (y1 + y2) / 2.0

    def accumulate(self):
        tot_pos = 0.0
        tot_neg = 0.0
        auc = 0.0

        idx = self._num_thresholds
        while idx >= 0:
            tot_pos_prev = tot_pos
            tot_neg_prev = tot_neg
            tot_pos += self._stat_pos[idx]
            tot_neg += self._stat_neg[idx]
            auc += self.trapezoid_area(
                tot_neg, tot_neg_prev, tot_pos, tot_pos_prev
            )
            idx -= 1

        return (
            auc / tot_pos / tot_neg if tot_pos > 0.0 and tot_neg > 0.0 else 0.0
        )

    def reset(self):
        _num_pred_buckets = self._num_thresholds + 1
        self._stat_pos = np.zeros(_num_pred_buckets)
        self._stat_neg = np.zeros(_num_pred_buckets)

    def name(self):
        return self._name



import os
import yaml

commit_message = os.getenv("CI_COMMIT_TITLE", "")
template_file = "test/data/template.yaml"
search_dirs = ["buildin/cv/classification", "buildin/cv/detection", "buildin/cv/segmentation", "buildin/cv/other", "buildin/nlp/LanguageModeling", "buildin/nlp/SpeechSynthesis", "buildin/nlp/SpeechRecognition", "buildin/nlp/Recommendation"]
dirs = []
for search_dir in search_dirs:
    s_dir = os.listdir(search_dir)
    for d in s_dir:
        dir = os.path.join(search_dir, d)
        # MagicMind r1.7 and later versions do not support TensorFlow Parser.
        if os.path.isdir(dir) and not d.endswith('_tensorflow'):
            dirs.append(dir)

with open(template_file, "r") as f:
    cfg = yaml.load(f.read(), Loader=yaml.FullLoader)

for dir in dirs:
    project_name = os.path.split(dir)[-1]

    #Check if CI_COMMIT_TITLE contains "-!project_name"
    if "-!" + project_name in commit_message:
        #If included, skip the current network
        continue

# If "-" not in commit message or "-skip_ci" in commit message, ci will not be executed
    cfg[f"{project_name}"] = {
        "extends": ".network_test",
        "variables": {
           "TEST_PROJ_DIR": f"{dir}"
        },
        "rules": [
            {"if": f"$CI_COMMIT_TITLE =~ /-{project_name}/"},
            {"if": "$CI_COMMIT_TITLE =~ /-ci_test/"}
        ]
    }

with open("jobs.yml", "w", encoding="utf-8") as f:
    yaml.dump(data=cfg, stream=f, allow_unicode=True)



import numpy as np
import magicmind.python.runtime as mm
import cv2
import os
import glob
import math
import torch
from PIL import Image
from magicmind.python.common.types import get_datatype_by_numpy
from typing import List

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, scale: float, max_samples: int, img_dir: str):
        super().__init__()
        print(img_dir)
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        t = glob.glob(img_dir + '/*.JPEG')
        self.mean = [0, 0, 0]
        self.std = [1.0, 1.0, 1.0]
        self.scale_ = scale
        self.shape_ = shape
        self.data_paths_ += t
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0

    def get_shape(self):
        return self.shape_
    
    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess(self, pil_img, scale):
        w, h = pil_img.size
        newW, newH = int(scale * w), int(scale * h)
        pil_img = pil_img.resize((newW, newH), resample = Image.BICUBIC)
        img = np.asarray(pil_img)
        img = img.astype(np.float32)
        img = np.transpose(img, (2, 0, 1))#hwc->chw
        img = np.expand_dims(img, 0)
        img = img / 255
        return img
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = Image.open(self.data_paths_[i])
            img = self.preprocess(img, self.scale_) 
            imgs.append(img)
        return np.ascontiguousarray(np.concatenate(tuple(imgs),axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()



import torch
import os
import transformers
from transformers import AutoModelForQuestionAnswering
import argparse
import warnings
warnings.filterwarnings("ignore")

parser = argparse.ArgumentParser()
parser.add_argument("--model_path", "--model_path", type=str, default="../data/models/pytorch_bert_base_cased_squad", help="model_path")
parser.add_argument("--pt_model", "--pt_model", type=str, default="../data/models/bert_squad_pytorch.pt", help="bert_squad_pytorch pt")
parser.add_argument("--batch_size", "--batch_size", type=int, default=16, help="batch_size")
parser.add_argument("--max_seq_length", "--max_seq_length", type=int, default=128, help="max_seq_length")

if __name__ == "__main__":
    args = parser.parse_args()
 
    pt_model = AutoModelForQuestionAnswering.from_pretrained(args.model_path, from_tf = False, config = None, cache_dir = None)
    pt_model.load_state_dict(torch.load(os.path.join(args.model_path, "pytorch_model.bin"), map_location = 'cpu'))
    pt_model.eval()

    tokens = torch.randint(0, 1, (args.batch_size, args.max_seq_length))
    segments = torch.randint(0, 1, (args.batch_size, args.max_seq_length))
    mask = torch.randint(0, 1, (args.batch_size, args.max_seq_length))

    # 保存模型文件
    torch.jit.save(torch.jit.trace(pt_model, (tokens, segments, mask)), args.pt_model)
    print("successfully save pt")



import json
import os
import numpy as np
import cv2
import argparse
import prettytable
ROOT_DIR = os.path.abspath(os.path.join(os.path.split(__file__)[0], ""))

parser = argparse.ArgumentParser()
parser.add_argument("--model", type=str)
parser.add_argument("--output_file", type=str)
parser.add_argument("--output_ok_file", type=str)
parser.add_argument("--metric", type=str, choices = ["normal", "tacotron"], default="normal")
report_file = "%s/perf_report.csv"%ROOT_DIR

# top1andtop5 compare
def compare_perf(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for line1 in lines1:
        if "Throughput (qps)" in line1:
            new_qps = float(line1.split(":")[1].strip())
    for line2 in lines2:
        if "Throughput (qps)" in line2:
            old_qps = float(line2.split(":")[1].strip()) 
    result = new_qps / old_qps
    return result

def compare_perf_tacotron(file1, file2):
    with open(file1, "r") as f:
        lines1 = f.readlines()
    with open(file2, "r") as f:
        lines2 = f.readlines()
    for line1 in lines1:
        if "tacotron2_items_per_sec average" in line1:
            new_qps = float(line1.split("=")[1].strip())
    for line2 in lines2:
        if "tacotron2_items_per_sec average" in line2:
            old_qps = float(line2.split("=")[1].strip())
    result = new_qps / old_qps
    return result

def summary(testcase_name, file, result):
    header = ["testcase", "file", "status", "similar"]
    show_tb = prettytable.PrettyTable()
    show_tb.align = "l"
    show_tb.field_names = header
    if os.path.exists(report_file):
        with open(report_file, "r") as f:
            tb = prettytable.from_csv(f)
            tb.align = "l"
    else:
        tb = prettytable.PrettyTable()
        tb.field_names = header
        tb.align = "l"

    if isinstance(result, float):
        status = "pass" 
        if result >= 0.95 and result <= 1.05: 
            status = "pass"
        elif result > 1.05:
            status = "option"
        else:
            status = "fail"
    else:
        status = "fail"
    
    show_tb.add_row ([testcase_name, file.split("/")[-1], status, result])
    tb.add_row ([testcase_name, file.split("/")[-1], status, result])
    
    print(show_tb)
    with open(report_file, "w") as f:
        f.write(tb.get_csv_string())

if __name__ == "__main__":
    args = parser.parse_args()
    testcase_name = args.model    
    output_file = os.path.join(args.output_file)
    output_ok_file = os.path.join(args.output_ok_file)
    if not os.path.exists(output_file):
        result = "file: %s not exists."%output_file
    elif not os.path.exists(output_ok_file):
        result = "file: %s not exists."%output_ok_file
    elif args.metric == "tacotron":
        result = compare_perf_tacotron(output_file, output_ok_file)
    else:
        result = compare_perf(output_file, output_ok_file)
    summary(testcase_name, args.output_file, result)



import torch
from torch.utils.data import DataLoader, Dataset
import transformers
from transformers import BertTokenizer
import csv
from itertools import islice
import os

class MyDataSet(Dataset):
    def __init__(self, corpus_file):
        self.data = []
        with open(corpus_file, 'r', encoding='utf-8') as f:
            reader = csv.reader(f)
            for item in islice(reader, 1, None): # 忽略表头
                item = item[0].split("\t")
                label, text = item[0], item[1]
                self.data.append((text, label))

    def __getitem__(self, index):
        return {"text": self.data[index][0], "label": int(self.data[index][1])}

    def __len__(self):
        return len(self.data)

def preprocess(batch_size, max_seq_length):
    ###preprocess data
    model_path = os.environ.get('MODEL_PATH')+ "/chinese-roberta-wwm-ext-chnsenticorp"
    dataset_path = os.environ.get('CHNSENTICORP_DATASETS_PATH')+ "/test.tsv"
    tokenizer = BertTokenizer.from_pretrained(model_path, do_lower_case = False)
    dataset = MyDataSet(dataset_path)
    eval_dataloader = DataLoader(dataset, batch_size = batch_size, shuffle = True, drop_last = True)
    print("Num examples = ", len(dataset))
    print("Batch size = ", batch_size)
    print("Iterations = ", len(eval_dataloader))
    return tokenizer, eval_dataloader



import magicmind.python.runtime as mm
from batchgenerators.utilities.file_and_folder_operations import *
from batchgenerators.augmentations.utils import pad_nd_image
import argparse
import numpy as np
import torch
from nnunet.inference.segmentation_export import save_segmentation_nifti_from_softmax
from nnunet.evaluation.evaluator import evaluate_folder
from preprocess import preprocess, compute_steps_for_sliding_window

parser = argparse.ArgumentParser()
parser.add_argument('--device_id', dest = 'device_id', default = 0,
            type = int, help = 'mlu device id, used for calibration')
parser.add_argument("--magicmind_model", type=str, default="magicmind_model", help="magicmind_model")
parser.add_argument("--model_path",
                    type=str,
                    help="nnUNet_trained_models/nnUNet/2d/Task002_Heart/nnUNetTrainerV2__nnUNetPlansv2.1",
                    required=True)
parser.add_argument("--data_folder",
                    type=str,
                    help="nnUNet_raw_data_base/nnUNet_raw_data/Task002_Heart/imagesTr",
                    required=True)
parser.add_argument("--output_folder",
                    type=str,
                    default="../data/results",
                    help="forder for saving results.")
parser.add_argument("--ref_folder",
                    type=str,
                    help="nnUNet_raw_data_base/nnUNet_raw_data/Task02_Heart/labelsTr",
                    required=True)

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind model first!!!")
        exit()
    model = mm.Model()
    model.deserialize_from_file(args.magicmind_model)

    pad_kwargs = {'constant_values': 0}
    pad_border_mode = 'constant'
    step_size = 0.5
    num_classes = 2
    num_results = 4
    regions_class_order = None
    preprocessor, list_of_lists, output_files, target_spacing, patch_size, inference_apply_nonlin, transpose_forward, transpose_backward, interpolation_order, force_separate_z, interpolation_order_z= preprocess(args)
    with mm.System():
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        context = engine.create_i_context()
        queue = dev.create_queue()
        assert queue != None

        inputs = context.create_inputs()
        outputs = []

        with torch.no_grad():
            for i, file_list in enumerate(list_of_lists):
                output_filename = output_files[i]
                d, s, dct = preprocessor.preprocess_test_case(file_list, target_spacing)
                # _internal_predict_3D_2Dconv_tiled
                softmaxes = []
                predicted_segmentation = []
                softmax_pred = []
                for s in range(d.shape[1]):
                    # _internal_predict_2D_2Dconv_tiled
                    assert len(d[:, s].shape) == 3, "x must be (c, x, y)"
                    data, slicer = pad_nd_image(d[:, s], patch_size, pad_border_mode, pad_kwargs, True,
                                                None)
                    data_shape = data.shape  # still c, x, y
                    steps = compute_steps_for_sliding_window(patch_size, data_shape[1:], step_size)
                    num_tiles = len(steps[0]) * len(steps[1])
                    add_for_nb_of_preds = np.ones(data.shape[1:], dtype=np.float32)
                    aggregated_results = np.zeros([num_classes] + list(data.shape[1:]),
                                                dtype=np.float32)
                    aggregated_nb_of_predictions = np.zeros([num_classes] + list(data.shape[1:]),
                                                            dtype=np.float32)

                    for step_x in steps[0]:
                        lb_x = step_x
                        ub_x = step_x + patch_size[0]
                        for step_y in steps[1]:
                            lb_y = step_y
                            ub_y = step_y + patch_size[1]

                            x = torch.from_numpy(data[None, :, lb_x:ub_x, lb_y:ub_y])
                            result_torch = torch.zeros([x.shape[0], num_classes] + list(x.shape[2:]),
                                                    dtype=torch.float)
                            x_1 = torch.flip(x, (3, ))
                            x_2 = torch.flip(x, (2, ))
                            x_3 = torch.flip(x, (3, 2))
                            mirrors_input = torch.cat([x,x_1,x_2,x_3], dim=0)
                            pre_data = mirrors_input.numpy()
                            pre_data = pre_data.transpose([0, 2, 3, 1])
                            for k in range(len([pre_data])):
                                inputs[k].from_numpy([pre_data][k])
                                inputs[k].to(dev)
                            status =context.enqueue(inputs, outputs, queue)
                            queue.sync()
                            if not status.ok():
                                print("status is not ok!")
                            result = []
                            for out in outputs:
                                if isinstance(out, list):
                                    for t in out:
                                        result.append(t.asnumpy())
                                else:
                                    result.append(out.asnumpy())
                            pred_np = result[0].transpose([0,3,1,2])
                            for m in range(num_results):
                                y = torch.from_numpy(pred_np[m:m+1,:,:,:])
                                pred = inference_apply_nonlin(y)
                                if m == 0:
                                    result_torch += 1 / num_results * pred
                                elif m == 1:
                                    result_torch += 1 / num_results * torch.flip(pred, (3, ))
                                elif m == 2:
                                    result_torch += 1 / num_results * torch.flip(pred, (3, ))
                                else:
                                    result_torch += 1 / num_results * torch.flip(pred, (3, 2))
                            # _internal_predict_2D_2Dconv_tiled
                            predicted_patch = result_torch[0].detach().cpu().numpy()
                            aggregated_results[:, lb_x:ub_x, lb_y:ub_y] += predicted_patch
                            aggregated_nb_of_predictions[:, lb_x:ub_x, lb_y:ub_y] += add_for_nb_of_preds
                    # we reverse the padding here (remeber that we padded the input to be at least as large as the patch size
                    slicer = tuple([
                        slice(0, aggregated_results.shape[i])
                        for i in range(len(aggregated_results.shape) - (len(slicer) - 1))
                    ] + slicer[1:])
                    aggregated_results = aggregated_results[slicer]
                    aggregated_nb_of_predictions = aggregated_nb_of_predictions[slicer]

                    # computing the class_probabilities by dividing the aggregated result with result_numsamples
                    class_probabilities = aggregated_results / aggregated_nb_of_predictions

                    if regions_class_order is None:
                        pred_seg = class_probabilities.argmax(0)
                    else:
                        class_probabilities_here = class_probabilities
                        pred_seg = np.zeros(class_probabilities_here.shape[1:], dtype=np.float32)
                        for i, c in enumerate(regions_class_order):
                            pred_seg[class_probabilities_here[i] > 0.5] = c

                    predicted_segmentation.append(pred_seg[None])
                    softmax_pred.append(class_probabilities[None])
                predicted_segmentation = np.vstack(predicted_segmentation)
                softmax_pred = np.vstack(softmax_pred).transpose((1, 0, 2, 3))
                softmaxes.append(softmax_pred)
                softmax = sum(softmaxes)
                if transpose_forward is not None:
                    softmax = softmax.transpose([0] + [i + 1 for i in transpose_backward])
                npz_file = None
                region_class_order = None
                bytes_per_voxel = 4
                save_segmentation_nifti_from_softmax(softmax, output_filename, dct, interpolation_order, \
                    region_class_order, None, None, npz_file, None, force_separate_z, interpolation_order_z)

    evaluate_folder(folder_with_gts=args.ref_folder,
                    folder_with_predictions=args.output_folder,
                    labels=[1])

    with open(args.output_folder + '/summary.json', 'r') as f:
        import json
        jstr = json.load(f)
        print('mean acc:', jstr['results']['mean'])
        print('see', args.output_folder + '/summary.json', 'for detail.')




#
# Copyright (c) 2021, NVIDIA CORPORATION. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
""" from https://github.com/keithito/tacotron """

'''
Defines the set of symbols used in text input to the model.

The default is a set of ASCII characters that works well for English or text that has been run through Unidecode. For other data, you can modify _characters. See TRAINING_DATA.md for details. '''
from . import cmudict

_pad        = '_'
_punctuation = '!\'(),.:;? '
_special = '-'
_letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'

# Prepend "@" to ARPAbet symbols to ensure uniqueness (some are the same as uppercase letters):
_arpabet = ['@' + s for s in cmudict.valid_symbols]

# Export all symbols:
symbols = [_pad] + list(_special) + list(_punctuation) + list(_letters) + _arpabet



import torch
import torchvision
from torchvision.models.swin_transformer import swin_t, Swin_T_Weights

weights = Swin_T_Weights.IMAGENET1K_V1
model = swin_t(weights = weights)
model.eval()
 
dummy_input = torch.randn(1, 3, 224, 224, device="cpu")
torch.onnx.export(model, dummy_input, "../data/models/swin.onnx",
                  input_names = ['images'],
                  opset_version = 11,
                  output_names = ['output'],
                  dynamic_axes={"images":{0:"batch_size"},'output':{0:"batch_size"}}
                  )



import argparse
import logging
import os, sys

import torch
sys.path.append("Pytorch-UNet")
from unet import UNet

MODEL_PATH = os.getenv("MODEL_PATH")

def get_args():
    parser = argparse.ArgumentParser(description='Predict masks from input images')
    parser.add_argument('--model', '-m', default= '../data/models/unet_carvana_scale0.5_epoch2.pth', metavar='FILE',
                        help='Specify the file in which the model is stored')
    parser.add_argument('--output_model', '-o', default= '../data/models/unet_carvana_scale0.5_epoch2_trace.pt', metavar='FILE',
                        help='Specify the file in which the model is stored')
    parser.add_argument('--bilinear', action='store_true', default=False, help='Use bilinear upsampling')
    parser.add_argument('--classes', '-c', type=int, default=2, help='Number of classes')
    return parser.parse_args()

if __name__ == '__main__':
    args = get_args()
    logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')
    tarce_name = args.output_model
    
    net = UNet(n_channels=3, n_classes=args.classes, bilinear=args.bilinear)
    device = torch.device('cpu')
    logging.info(f'Loading model {args.model}')

    net.to(device=device) 
    state_dict = torch.load(args.model, map_location=device)
    mask_values = state_dict.pop('mask_values', [0, 1])
    net.load_state_dict(state_dict)
    logging.info('Model loaded!')

    net.eval()
    trace_input = torch.randn(1, 3, 640, 959, dtype = torch.float)
    trace_model = torch.jit.trace(net, trace_input)
    torch.jit.save(trace_model, tarce_name)
    logging.info('torch.jit.trace model saved in ' + tarce_name)



import argparse
import magicmind.python.runtime as mm
from calibrator import CalibData
from preprocess import preprocess, imagenet_dataset
import time
import numpy as np

# import common components
from framework_parser import OnnxParser
from common_calibrate import common_calibrate
from build_param import get_argparser 
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger

log = Logger()

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()
    return network

def calibrate(args, network : mm.Network, config : mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    batch_size = args.input_dims[0][0]
    dst_size = (args.input_dims[0][2],args.input_dims[0][3])
    dataset = imagenet_dataset(val_txt =args.label_file, image_file_path = args.image_dir, count=batch_size)
    sample_data = []
    for data, label in dataset:
        data = preprocess(data, dst_size, transpose=True)
        sample_data.append(data)
    calib_data = CalibData([np.array(sample_data)])
    common_calibrate(args, network, config, calib_data)  

def get_args():
    # get common argparser,here is pytorch_parser
    arg_parser = get_argparser()

    # add custom args belonging to the current net.
    arg_parser.add_argument(
        "--image_dir",
        dest="image_dir",
        type=str,
        default="ILSVRC2012_DATASETS_PATH",
        help="ILSVRC2012_DATASETS_PATH",
    )
    arg_parser.add_argument(
        "--label_file", "--label_file",  
        type=str, default="/path/to/modelzoo/datasets/imageNet2012/labels.txt",
        help="imagenet val label txt")
    return arg_parser.parse_args()

def main():
    begin_time = time.time()
    # get net args
    args = get_args()
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)

    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)
    log.info("Build model...")
    # 生成模型并导出
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == "__main__":
    main()




import numpy as np
import magicmind.python.runtime as mm
from magicmind.python.common.types import get_datatype_by_numpy
from typing import List

class CalibData(mm.CalibDataInterface):
    def __init__(self, data_list: List[np.ndarray]):
        super(CalibData, self).__init__()
        self._data_list = data_list
        self._cur_idx = 0
        status = self.reset()
        assert status.ok(), str(status)

    def next(self):
        if self._cur_idx >= len(self._data_list):
            return mm.Status(mm.Code.OUT_OF_RANGE, "No more data.")
        self._data = np.ascontiguousarray(self._data_list[self._cur_idx])
        self._data_shape = mm.Dims(self._data.shape)
        self._data_type = get_datatype_by_numpy(self._data.dtype)
        self._cur_idx += 1
        return mm.Status()

    def get_shape(self):
        return self._data_shape

    def get_data_type(self):
        return self._data_type

    def get_sample(self):
        return self._data

    def reset(self):
        self._cur_idx = 0
        return mm.Status()



import os
import prettytable
import sys
import csv
from glob import glob
import shutil

bench_csv = "benchmark.csv"
bench_ok_csv = "benchmark_ok.csv"

if os.path.exists(bench_csv):
    os.remove(bench_csv)
    
if os.path.exists(bench_ok_csv):
    os.remove(bench_ok_csv)

all_file = glob("**/benchmark.csv", recursive=True)
all_ok_file = glob("**/benchmark_ok.csv", recursive=True)

def load_csv(csv_file):
    with open(csv_file,'r') as f:
        reader = csv.DictReader(f)
        result = [x for x in reader]
    return result

bench = [load_csv(x)[0] for x in all_file]
with open(bench_csv, "w") as f:
    csv_writter = csv.DictWriter(f, bench[0].keys())
    csv_writter.writeheader()
    csv_writter.writerows(bench)

bench_ok = [load_csv(x)[0] for x in all_ok_file]
with open(bench_ok_csv, "w") as f:
    csv_writter = csv.DictWriter(f, bench_ok[0].keys())
    csv_writter.writeheader()
    csv_writter.writerows(bench_ok)




from magicmind.python.runtime import (
    ModelKind,
    IDetectionOutputAlgo,
    Dims,
    DataType,
    Layout,
)
import numpy as np


def append_yolov5_detect(network, conf, iou, max_det, yolov5_h, yolov5_w):
    # 使用permute算子将三个卷积层输出由NCHW(PyTorch模型数据均为NCHW摆放顺序)转为NHWC摆放顺序
    perms = [0, 2, 3, 1]  # 0 : N, 1 : C, 2 : H, 3 : W
    const_node = network.add_i_const_node(
        DataType.INT32, Dims([len(perms)]), np.array(perms, dtype=np.int32)
    )
    output_tensors = []
    for i in range(network.get_output_count()):
        # 添加premute算子做NCHW到NHWC的转换
        tensor = network.get_output(i)
        permute_node = network.add_i_permute_node(tensor, const_node.get_output(0))
        output_tensors.append(permute_node.get_output(0))
    output_count = network.get_output_count()
    for i in range(output_count):
        # 去掉原网络输出tensor标志
        network.unmark_output(network.get_output(0))

    # anchors，按原始3个yolo层顺序填写
    bias_buffer = [
        10,
        13,
        16,
        30,
        33,
        23,
        30,
        61,
        62,
        45,
        59,
        119,
        116,
        90,
        156,
        198,
        373,
        326,
    ]
    bias_node = network.add_i_const_node(
        DataType.FLOAT32,
        Dims([len(bias_buffer)]),
        np.array(bias_buffer, dtype=np.float32),
    )
    detect_out = network.add_i_detection_output_node(
        output_tensors, bias_node.get_output(0)
    )
    detect_out.set_algo(IDetectionOutputAlgo.YOLOV5)
    detect_out.set_confidence_thresh(conf)
    detect_out.set_nms_thresh(iou)
    detect_out.set_scale(1.0)
    detect_out.set_num_coord(4)
    detect_out.set_num_class(80)
    detect_out.set_num_entry(5)
    detect_out.set_num_anchor(3)
    detect_out.set_num_box_limit(max_det)
    detect_out.set_image_shape(yolov5_h, yolov5_w)
    detect_out.set_layout(Layout.NONE, Layout.NONE)
    # 将detect_out层输出标记为网络输出
    detection_output_count = detect_out.get_output_count()
    for i in range(detection_output_count):
        network.mark_output(detect_out.get_output(i))

    return network



# Copyright (c) 2020 Mobvoi Inc. (authors: Binbin Zhang, Chao Yang)
# Copyright (c) 2021 Jinsong Pan
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import codecs
import copy
import logging
import random

import numpy as np
import torch
import torchaudio
import torchaudio.compliance.kaldi as kaldi
import torchaudio.sox_effects as sox_effects
import yaml
from PIL import Image
from PIL.Image import BICUBIC
from torch.nn.utils.rnn import pad_sequence
from torch.utils.data import Dataset, DataLoader

import wenet.dataset.kaldi_io as kaldi_io
from wenet.dataset.wav_distortion import distort_wav_conf
from wenet.utils.common import IGNORE_ID

torchaudio.set_audio_backend("sox_io")


def _spec_augmentation(x,
                       warp_for_time=False,
                       num_t_mask=2,
                       num_f_mask=2,
                       max_t=50,
                       max_f=10,
                       max_w=80):
    """ Deep copy x and do spec augmentation then return it

    Args:
        x: input feature, T * F 2D
        num_t_mask: number of time mask to apply
        num_f_mask: number of freq mask to apply
        max_t: max width of time mask
        max_f: max width of freq mask
        max_w: max width of time warp

    Returns:
        augmented feature
    """
    y = np.copy(x)
    max_frames = y.shape[0]
    max_freq = y.shape[1]

    # time warp
    if warp_for_time and max_frames > max_w * 2:
        center = random.randrange(max_w, max_frames - max_w)
        warped = random.randrange(center - max_w, center + max_w) + 1

        left = Image.fromarray(x[:center]).resize((max_freq, warped), BICUBIC)
        right = Image.fromarray(x[center:]).resize(
            (max_freq, max_frames - warped), BICUBIC)
        y = np.concatenate((left, right), 0)
    # time mask
    for i in range(num_t_mask):
        start = random.randint(0, max_frames - 1)
        length = random.randint(1, max_t)
        end = min(max_frames, start + length)
        y[start:end, :] = 0
    # freq mask
    for i in range(num_f_mask):
        start = random.randint(0, max_freq - 1)
        length = random.randint(1, max_f)
        end = min(max_freq, start + length)
        y[:, start:end] = 0
    return y


def _spec_substitute(x, max_t=20, num_t_sub=3):
    """ Deep copy x and do spec substitute then return it

    Args:
        x: input feature, T * F 2D
        max_t: max width of time substitute
        num_t_sub: number of time substitute to apply

    Returns:
        augmented feature
    """
    y = np.copy(x)
    max_frames = y.shape[0]
    for i in range(num_t_sub):
        start = random.randint(0, max_frames - 1)
        length = random.randint(1, max_t)
        end = min(max_frames, start + length)
        # only substitute the earlier time chosen randomly for current time
        pos = random.randint(0, start)
        y[start:end, :] = y[start - pos:end - pos, :]
    return y


def _waveform_distortion(waveform, distortion_methods_conf):
    """ Apply distortion on waveform

    This distortion will not change the length of the waveform.

    Args:
        waveform: numpy float tensor, (length,)
        distortion_methods_conf: a list of config for ditortion method.
            a method will be randomly selected by 'method_rate' and
            apply on the waveform.

    Returns:
        distorted waveform.
    """
    r = random.uniform(0, 1)
    acc = 0.0
    for distortion_method in distortion_methods_conf:
        method_rate = distortion_method['method_rate']
        acc += method_rate
        if r < acc:
            distortion_type = distortion_method['name']
            distortion_conf = distortion_method['params']
            point_rate = distortion_method['point_rate']
            return distort_wav_conf(waveform, distortion_type, distortion_conf,
                                    point_rate)
    return waveform


# add speed perturb when loading wav
# return augmented, sr
def _load_wav_with_speed(wav_file, speed):
    """ Load the wave from file and apply speed perpturbation

    Args:
        wav_file: input feature, T * F 2D

    Returns:
        augmented feature
    """
    if speed == 1.0:
        wav, sr = torchaudio.load(wav_file)
    else:
        sample_rate = torchaudio.backend.sox_io_backend.info(
            wav_file).sample_rate
        # get torchaudio version
        ta_no = torchaudio.__version__.split(".")
        ta_version = 100 * int(ta_no[0]) + 10 * int(ta_no[1])

        if ta_version < 80:
            # Note: deprecated in torchaudio>=0.8.0
            E = sox_effects.SoxEffectsChain()
            E.append_effect_to_chain('speed', speed)
            E.append_effect_to_chain("rate", sample_rate)
            E.set_input_file(wav_file)
            wav, sr = E.sox_build_flow_effects()
        else:
            # Note: enable in torchaudio>=0.8.0
            wav, sr = sox_effects.apply_effects_file(
                wav_file,
                [['speed', str(speed)], ['rate', str(sample_rate)]])

    return wav, sr


def _extract_feature(batch, speed_perturb, wav_distortion_conf,
                     feature_extraction_conf):
    """ Extract acoustic fbank feature from origin waveform.

    Speed perturbation and wave amplitude distortion is optional.

    Args:
        batch: a list of tuple (wav id , wave path).
        speed_perturb: bool, whether or not to use speed pertubation.
        wav_distortion_conf: a dict , the config of wave amplitude distortion.
        feature_extraction_conf:a dict , the config of fbank extraction.

    Returns:
        (keys, feats, labels)
    """
    keys = []
    feats = []
    lengths = []
    wav_dither = wav_distortion_conf['wav_dither']
    wav_distortion_rate = wav_distortion_conf['wav_distortion_rate']
    distortion_methods_conf = wav_distortion_conf['distortion_methods']
    if speed_perturb:
        speeds = [1.0, 1.1, 0.9]
        weights = [1, 1, 1]
        speed = random.choices(speeds, weights, k=1)[0]
        # speed = random.choice(speeds)
    for i, x in enumerate(batch):
        try:
            wav = x[1]
            value = wav.strip().split(",")
            # 1 for general wav.scp, 3 for segmented wav.scp
            assert len(value) == 1 or len(value) == 3
            wav_path = value[0]
            sample_rate = torchaudio.backend.sox_io_backend.info(
                wav_path).sample_rate
            if 'resample' in feature_extraction_conf:
                resample_rate = feature_extraction_conf['resample']
            else:
                resample_rate = sample_rate
            if speed_perturb:
                if len(value) == 3:
                    logging.error(
                        "speed perturb does not support segmented wav.scp now")
                assert len(value) == 1
                waveform, sample_rate = _load_wav_with_speed(wav_path, speed)
            else:
                # value length 3 means using segmented wav.scp
                # incluede .wav, start time, end time
                if len(value) == 3:
                    start_frame = int(float(value[1]) * sample_rate)
                    end_frame = int(float(value[2]) * sample_rate)
                    waveform, sample_rate = torchaudio.backend.sox_io_backend.load(
                        filepath=wav_path,
                        num_frames=end_frame - start_frame,
                        frame_offset=start_frame)
                else:
                    waveform, sample_rate = torchaudio.load(wav_path)
            waveform = waveform * (1 << 15)
            if resample_rate != sample_rate:
                waveform = torchaudio.transforms.Resample(
                    orig_freq=sample_rate, new_freq=resample_rate)(waveform)

            if wav_distortion_rate > 0.0:
                r = random.uniform(0, 1)
                if r < wav_distortion_rate:
                    waveform = waveform.detach().numpy()
                    waveform = _waveform_distortion(waveform,
                                                    distortion_methods_conf)
                    waveform = torch.from_numpy(waveform)
            mat = kaldi.fbank(
                waveform,
                num_mel_bins=feature_extraction_conf['mel_bins'],
                frame_length=feature_extraction_conf['frame_length'],
                frame_shift=feature_extraction_conf['frame_shift'],
                dither=wav_dither,
                energy_floor=0.0,
                sample_frequency=resample_rate)
            mat = mat.detach().numpy()
            feats.append(mat)
            keys.append(x[0])
            lengths.append(mat.shape[0])
        except (Exception) as e:
            print(e)
            logging.warn('read utterance {} error'.format(x[0]))
            pass
    # Sort it because sorting is required in pack/pad operation
    order = np.argsort(lengths)[::-1]
    sorted_keys = [keys[i] for i in order]
    sorted_feats = [feats[i] for i in order]
    labels = [x[2].split() for x in batch]
    labels = [np.fromiter(map(int, x), dtype=np.int32) for x in labels]
    sorted_labels = [labels[i] for i in order]
    return sorted_keys, sorted_feats, sorted_labels


def _load_feature(batch):
    """ Load acoustic feature from files.

    The features have been prepared in previous step, usualy by Kaldi.

    Args:
        batch: a list of tuple (wav id , feature ark path).

    Returns:
        (keys, feats, labels)
    """
    keys = []
    feats = []
    lengths = []
    for i, x in enumerate(batch):
        try:
            mat = kaldi_io.read_mat(x[1])
            feats.append(mat)
            keys.append(x[0])
            lengths.append(mat.shape[0])
        except (Exception):
            # logging.warn('read utterance {} error'.format(x[0]))
            pass
    # Sort it because sorting is required in pack/pad operation
    order = np.argsort(lengths)[::-1]
    sorted_keys = [keys[i] for i in order]
    sorted_feats = [feats[i] for i in order]
    labels = [x[2].split() for x in batch]
    labels = [np.fromiter(map(int, x), dtype=np.int32) for x in labels]
    sorted_labels = [labels[i] for i in order]
    return sorted_keys, sorted_feats, sorted_labels


class CollateFunc(object):
    """ Collate function for AudioDataset
    """
    def __init__(
        self,
        feature_dither=0.0,
        speed_perturb=False,
        spec_aug=False,
        spec_aug_conf=None,
        spec_sub=False,
        spec_sub_conf=None,
        raw_wav=True,
        feature_extraction_conf=None,
        wav_distortion_conf=None,
    ):
        """
        Args:
            raw_wav:
                    True if input is raw wav and feature extraction is needed.
                    False if input is extracted feature
        """
        self.wav_distortion_conf = wav_distortion_conf
        self.feature_extraction_conf = feature_extraction_conf
        self.spec_aug = spec_aug
        self.feature_dither = feature_dither
        self.speed_perturb = speed_perturb
        self.raw_wav = raw_wav
        self.spec_aug_conf = spec_aug_conf
        self.spec_sub = spec_sub
        self.spec_sub_conf = spec_sub_conf

    def __call__(self, batch):
        assert (len(batch) == 1)
        if self.raw_wav:
            keys, xs, ys = _extract_feature(batch[0], self.speed_perturb,
                                            self.wav_distortion_conf,
                                            self.feature_extraction_conf)

        else:
            keys, xs, ys = _load_feature(batch[0])

        train_flag = True
        if ys is None:
            train_flag = False

        # optional feature dither d ~ (-a, a) on fbank feature
        # a ~ (0, 0.5)
        if self.feature_dither != 0.0:
            a = random.uniform(0, self.feature_dither)
            xs = [x + (np.random.random_sample(x.shape) - 0.5) * a for x in xs]

        # optinoal spec substitute
        if self.spec_sub:
            xs = [_spec_substitute(x, **self.spec_sub_conf) for x in xs]

        # optinoal spec augmentation
        if self.spec_aug:
            xs = [_spec_augmentation(x, **self.spec_aug_conf) for x in xs]

        # padding
        xs_lengths = torch.from_numpy(
            np.array([x.shape[0] for x in xs], dtype=np.int32))

        # pad_sequence will FAIL in case xs is empty
        if len(xs) > 0:
            xs_pad = pad_sequence([torch.from_numpy(x).float() for x in xs],
                                  True, 0)
        else:
            xs_pad = torch.Tensor(xs)
        if train_flag:
            ys_lengths = torch.from_numpy(
                np.array([y.shape[0] for y in ys], dtype=np.int32))
            if len(ys) > 0:
                ys_pad = pad_sequence([torch.from_numpy(y).int() for y in ys],
                                      True, IGNORE_ID)
            else:
                ys_pad = torch.Tensor(ys)
        else:
            ys_pad = None
            ys_lengths = None
        return keys, xs_pad, ys_pad, xs_lengths, ys_lengths


class AudioDataset(Dataset):
    def __init__(self,
                 data_file,
                 max_length=10240,
                 min_length=0,
                 token_max_length=200,
                 token_min_length=1,
                 batch_type='static',
                 batch_size=1,
                 max_frames_in_batch=0,
                 sort=True,
                 raw_wav=True):
        """Dataset for loading audio data.

        Attributes::
            data_file: input data file
                Plain text data file, each line contains following 7 fields,
                which is split by '\t':
                    utt:utt1
                    feat:tmp/data/file1.wav or feat:tmp/data/fbank.ark:30
                    feat_shape: 4.95(in seconds) or feat_shape:495,80(495 is in frames)
                    text:i love you
                    token: i <space> l o v e <space> y o u
                    tokenid: int id of this token
                    token_shape: M,N    # M is the number of token, N is vocab size
            max_length: drop utterance which is greater than max_length(10ms)
            min_length: drop utterance which is less than min_length(10ms)
            token_max_length: drop utterance which is greater than token_max_length,
                especially when use char unit for english modeling
            token_min_length: drop utterance which is less than token_max_length
            batch_type: static or dynamic, see max_frames_in_batch(dynamic)
            batch_size: number of utterances in a batch,
               it's for static batch size.
            max_frames_in_batch: max feature frames in a batch,
               when batch_type is dynamic, it's for dynamic batch size.
               Then batch_size is ignored, we will keep filling the
               batch until the total frames in batch up to max_frames_in_batch.
            sort: whether to sort all data, so the utterance with the same
               length could be filled in a same batch.
            raw_wav: use raw wave or extracted featute.
                if raw wave is used, dynamic waveform-level augmentation could be used
                and the feature is extracted by torchaudio.
                if extracted featute(e.g. by kaldi) is used, only feature-level
                augmentation such as specaug could be used.
        """
        assert batch_type in ['static', 'dynamic']
        data = []

        # Open in utf8 mode since meet encoding problem
        with codecs.open(data_file, 'r', encoding='utf-8') as f:
            for line in f:
                arr = line.strip().split('\t')
                if len(arr) != 7:
                    continue
                key = arr[0].split(':')[1]
                tokenid = arr[5].split(':')[1]
                output_dim = int(arr[6].split(':')[1].split(',')[1])
                if raw_wav:
                    wav_path = ':'.join(arr[1].split(':')[1:])
                    duration = int(float(arr[2].split(':')[1]) * 1000 / 10)
                    data.append((key, wav_path, duration, tokenid))
                else:
                    feat_ark = ':'.join(arr[1].split(':')[1:])
                    feat_info = arr[2].split(':')[1].split(',')
                    feat_dim = int(feat_info[1].strip())
                    num_frames = int(feat_info[0].strip())
                    data.append((key, feat_ark, num_frames, tokenid))
                    self.input_dim = feat_dim
                self.output_dim = output_dim
        if sort:
            data = sorted(data, key=lambda x: x[2])
        valid_data = []
        for i in range(len(data)):
            length = data[i][2]
            token_length = len(data[i][3].split())
            # remove too lang or too short utt for both input and output
            # to prevent from out of memory
            if length > max_length or length < min_length:
                # logging.warn('ignore utterance {} feature {}'.format(
                #     data[i][0], length))
                pass
            elif token_length > token_max_length or token_length < token_min_length:
                pass
            else:
                valid_data.append(data[i])
        data = valid_data
        self.minibatch = []
        num_data = len(data)
        # Dynamic batch size
        if batch_type == 'dynamic':
            assert (max_frames_in_batch > 0)
            self.minibatch.append([])
            num_frames_in_batch = 0
            for i in range(num_data):
                length = data[i][2]
                num_frames_in_batch += length
                if num_frames_in_batch > max_frames_in_batch:
                    self.minibatch.append([])
                    num_frames_in_batch = length
                self.minibatch[-1].append((data[i][0], data[i][1], data[i][3]))
        # Static batch size
        else:
            cur = 0
            while cur < num_data:
                end = min(cur + batch_size, num_data)
                item = []
                for i in range(cur, end):
                    item.append((data[i][0], data[i][1], data[i][3]))
                self.minibatch.append(item)
                cur = end

    def __len__(self):
        return len(self.minibatch)

    def __getitem__(self, idx):
        return self.minibatch[idx]


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('type', help='config file')
    parser.add_argument('config_file', help='config file')
    parser.add_argument('data_file', help='input data file')
    args = parser.parse_args()

    with open(args.config_file, 'r') as fin:
        configs = yaml.load(fin, Loader=yaml.FullLoader)

    # Init dataset and data loader
    collate_conf = copy.copy(configs['collate_conf'])
    if args.type == 'raw_wav':
        raw_wav = True
    else:
        raw_wav = False
    collate_func = CollateFunc(**collate_conf, raw_wav=raw_wav)
    dataset_conf = configs.get('dataset_conf', {})
    dataset = AudioDataset(args.data_file, **dataset_conf, raw_wav=raw_wav)

    data_loader = DataLoader(dataset,
                             batch_size=1,
                             shuffle=True,
                             sampler=None,
                             num_workers=0,
                             collate_fn=collate_func)

    for i, batch in enumerate(data_loader):
        print(i)
        # print(batch[1].shape)



import os
import prettytable
import csv
import sys

if len(sys.argv) == 3:
    benchmark_csv = sys.argv[1]
    benchmark_ok_csv = sys.argv[2]
else:
    benchmark_csv = os.path.join(os.getenv("PROJ_ROOT_PATH"), "benchmark","benchmark.csv")
    benchmark_ok_csv = os.path.join(os.getenv("PROJ_ROOT_PATH"), "benchmark","benchmark_ok.csv")

if not os.path.exists(benchmark_csv):
    raise Exception(f"{benchmark_csv} not exists")
elif not os.path.exists(benchmark_ok_csv):
    raise Exception(f"{benchmark_ok_csv} not exists")

def load_csv(csv_file):
    with open(csv_file,'r') as f:
        reader = csv.DictReader(f)
        result = [x for x in reader]
    return result

def check(value1, value2, thresh = 0.05):
    margin = abs(value1 - value2) / abs(value1)
    if margin > thresh:
        return False
    else:
        return True

status_list = list()
tb = prettytable.PrettyTable()
benchmarks = load_csv(benchmark_csv)
benchmarks_ok = load_csv(benchmark_ok_csv)
tb.field_names = [x for x in benchmarks_ok[0].keys()]

for bench, bench_ok in zip(benchmarks, benchmarks_ok):
    check_ok = True
    err_msg = list()
    acc = eval(bench["eval"])
    acc_ok = eval(bench_ok["eval"])

    if not check(acc_ok, acc):
        check_ok = False

    if check_ok:
        status_list.append("pass")
    else:
        status_list.append("fail")
    
    value = [] 
    for k in bench_ok.keys():
        if k in ["fps", "eval"]:
            v = eval(bench[k])
            v_ok = eval(bench_ok[k])
            value.append("%.3f (%.3f)"%(v,v_ok))
        else:
            value.append(bench[k])
    tb.add_row(value)

tb.add_column("status",status_list)
tb.align = "c"
tb.max_width = 40
print(tb)

if "fail" in  status_list:
    exit(1)




import os
import prettytable

ROOT_DIR = os.path.abspath(os.path.join(os.path.split(__file__)[0], "../test/"))
image_dir_list = os.listdir(ROOT_DIR)
report_files = []
for image_dir in image_dir_list:
    if os.path.splitext(image_dir)[1] == ".csv":
        report_file = os.path.join(ROOT_DIR, image_dir)
        report_files.append(report_file)

for report_file in report_files:
    with open(report_file, "r") as f:
        tb = prettytable.from_csv(f)
    tb.align = "l"
    print(report_file)
    print(tb)





# Copyright 2019 Mobvoi Inc. All Rights Reserved.
# Author: binbinzhang@mobvoi.com (Binbin Zhang)

import logging
from contextlib import nullcontext
# if your python version < 3.7 use the below one
# from contextlib import suppress as nullcontext
import torch
from torch.nn.utils import clip_grad_norm_


class Executor:
    def __init__(self):
        self.step = 0

    def train(self, model, optimizer, scheduler, data_loader, device, writer,
              args, scaler):
        ''' Train one epoch
        '''
        model.train()
        clip = args.get('grad_clip', 50.0)
        log_interval = args.get('log_interval', 10)
        rank = args.get('rank', 0)
        epoch = args.get('epoch', 0)
        accum_grad = args.get('accum_grad', 1)
        is_distributed = args.get('is_distributed', True)
        use_amp = args.get('use_amp', False)
        logging.info('using accumulate grad, new batch size is {} times'
                     ' larger than before'.format(accum_grad))
        if use_amp:
            assert scaler is not None
        # A context manager to be used in conjunction with an instance of
        # torch.nn.parallel.DistributedDataParallel to be able to train
        # with uneven inputs across participating processes.
        if isinstance(model, torch.nn.parallel.DistributedDataParallel):
            model_context = model.join
        else:
            model_context = nullcontext
        num_seen_utts = 0
        with model_context():
            for batch_idx, batch in enumerate(data_loader):
                key, feats, target, feats_lengths, target_lengths = batch
                feats = feats.to(device)
                target = target.to(device)
                feats_lengths = feats_lengths.to(device)
                target_lengths = target_lengths.to(device)
                num_utts = target_lengths.size(0)
                if num_utts == 0:
                    continue
                context = None
                # Disable gradient synchronizations across DDP processes.
                # Within this context, gradients will be accumulated on module
                # variables, which will later be synchronized.
                if is_distributed and batch_idx % accum_grad != 0:
                    context = model.no_sync
                # Used for single gpu training and DDP gradient synchronization
                # processes.
                else:
                    context = nullcontext
                with context():
                    # autocast context
                    # The more details about amp can be found in
                    # https://pytorch.org/docs/stable/notes/amp_examples.html
                    with torch.cuda.amp.autocast(scaler is not None):
                        loss, loss_att, loss_ctc = model(
                            feats, feats_lengths, target, target_lengths)
                        loss = loss / accum_grad
                    if use_amp:
                        scaler.scale(loss).backward()
                    else:
                        loss.backward()

                num_seen_utts += num_utts
                if batch_idx % accum_grad == 0:
                    if rank == 0 and writer is not None:
                        writer.add_scalar('train_loss', loss, self.step)
                    # Use mixed precision training
                    if use_amp:
                        scaler.unscale_(optimizer)
                        grad_norm = clip_grad_norm_(model.parameters(), clip)
                        # Must invoke scaler.update() if unscale_() is used in
                        # the iteration to avoid the following error:
                        #   RuntimeError: unscale_() has already been called
                        #   on this optimizer since the last update().
                        # We don't check grad here since that if the gradient
                        # has inf/nan values, scaler.step will skip
                        # optimizer.step().
                        scaler.step(optimizer)
                        scaler.update()
                    else:
                        grad_norm = clip_grad_norm_(model.parameters(), clip)
                        if torch.isfinite(grad_norm):
                            optimizer.step()
                    optimizer.zero_grad()
                    scheduler.step()
                    self.step += 1
                if batch_idx % log_interval == 0:
                    lr = optimizer.param_groups[0]['lr']
                    log_str = 'TRAIN Batch {}/{} loss {:.6f} '.format(
                        epoch, batch_idx,
                        loss.item() * accum_grad)
                    if loss_att is not None:
                        log_str += 'loss_att {:.6f} '.format(loss_att.item())
                    if loss_ctc is not None:
                        log_str += 'loss_ctc {:.6f} '.format(loss_ctc.item())
                    log_str += 'lr {:.8f} rank {}'.format(lr, rank)
                    logging.debug(log_str)

    def cv(self, model, data_loader, device, args):
        ''' Cross validation on
        '''
        model.eval()
        rank = args.get('rank', 0)
        epoch = args.get('epoch', 0)
        log_interval = args.get('log_interval', 10)
        # in order to avoid division by 0
        num_seen_utts = 1
        total_loss = 0.0
        with torch.no_grad():
            for batch_idx, batch in enumerate(data_loader):
                key, feats, target, feats_lengths, target_lengths = batch
                feats = feats.to(device)
                target = target.to(device)
                feats_lengths = feats_lengths.to(device)
                target_lengths = target_lengths.to(device)
                num_utts = target_lengths.size(0)
                if num_utts == 0:
                    continue
                loss, loss_att, loss_ctc = model(feats, feats_lengths, target,
                                                 target_lengths)
                if torch.isfinite(loss):
                    num_seen_utts += num_utts
                    total_loss += loss.item() * num_utts
                if batch_idx % log_interval == 0:
                    log_str = 'CV Batch {}/{} loss {:.6f} '.format(
                        epoch, batch_idx, loss.item())
                    if loss_att is not None:
                        log_str += 'loss_att {:.6f} '.format(loss_att.item())
                    if loss_ctc is not None:
                        log_str += 'loss_ctc {:.6f} '.format(loss_ctc.item())
                    log_str += 'history loss {:.6f}'.format(total_loss /
                                                            num_seen_utts)
                    log_str += ' rank {}'.format(rank)
                    logging.debug(log_str)
        return total_loss, num_seen_utts



import cv2
import numpy as np
import os
from PIL import Image
import logging
from torchvision import transforms

def imagenet_dataset(
    val_txt,
    image_file_path,
    count=-1
):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    logging.info("%d pictures will be read." % len(lines))
    if len(lines) < count:
        print("infer pictures less than {}".format(count))
        return
    current_count = 0
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        img = Image.open(image_path)
        if img.mode != "RGB":
            img = img.convert("RGB")
        yield np.array(img), label.strip()
        current_count += 1
        if current_count >= count and count != -1:
            break

def preprocess(img, transpose, normalization):
    img = cv2.resize(img, (256, 256))
    h, w = img.shape[0], img.shape[1]
    y0 = (h - 224) // 2
    x0 = (w - 224) // 2
    img = img[y0 : y0+224, x0 : x0+224, :]
    if normalization:
        img = img / 255.
        img = (img - [0.485, 0.456, 0.406]) / [0.229, 0.224, 0.225]
    img = img.astype(np.float32)
    if not transpose:
        img = np.transpose(img, axes=[2, 0, 1])
    return img



#!/usr/bin/python
# encoding: utf-8

import random
import torch
from torch.utils.data import Dataset
from torch.utils.data import sampler
import torchvision.transforms as transforms
import lmdb
import six
import sys
from PIL import Image
import numpy as np
import io
import os

class lmdbDataset(Dataset):
    def __init__(self, root=None, transform=None, target_transform=None):
        self.root = root

        self.transform = transform
        self.target_transform = target_transform

        imagePathList = self.root + "/annotation.txt"
        with open(imagePathList, "r") as f:
            self.annotation = f.readlines()
        self.annotation.sort()
        self.nSamples = len(self.annotation) - 1
        print(self.nSamples)

    def open_lmdb(self):
        self.env = lmdb.open(
            self.root,
            max_readers=1,
            readonly=True,
            lock=False,
            readahead=False,
            meminit=False)
        if not self.env:
            print("cannot create lmdb from %s" % (self.root))
            sys.exit(0)

    def __len__(self):
        return self.nSamples

    def __getitem__(self, index):
        assert index <= len(self), "index range error"
        index += 1

        i = self.annotation[index]
        imagePath = i.split(" ")[0]
        label = i.split(" ")[1]
        if '\n' in label:
            label=label.strip("\n")
        imagePath = os.path.join(self.root + "/mnt/ramdisk/max/90kDICT32px/", imagePath)
        byteImgIO = io.BytesIO()
        byteImg = Image.open(imagePath)
        byteImg.save(byteImgIO, "PNG")
        byteImgIO.seek(0)
        byteImg = byteImgIO.read()
        dataBytesIO = io.BytesIO(byteImg)
        try:
            img = Image.open(dataBytesIO).convert("L")
        except IOError:
            print("Corrupted image for %d" % index)
            return self[index + 1]
        if self.transform is not None:
            img = self.transform(img)

        label_key = "label-%09d" % index
        label = str(label)
        if self.target_transform is not None:
            label = self.target_transform(label)
        return (img, label)

class resizeNormalize(object):
    def __init__(self, size, interpolation=Image.BILINEAR):
        self.size = size
        self.interpolation = interpolation
        self.toTensor = transforms.ToTensor()

    def __call__(self, img):
        img = img.resize(self.size, self.interpolation)
        img = self.toTensor(img)
        img.sub_(0.5).div_(0.5)
        return img


class randomSequentialSampler(sampler.Sampler):
    def __init__(self, data_source, batch_size):
        self.num_samples = len(data_source)
        self.batch_size = batch_size

    def __iter__(self):
        n_batch = len(self) // self.batch_size
        tail = len(self) % self.batch_size
        index = torch.LongTensor(len(self)).fill_(0)
        for i in range(n_batch):
            random_start = random.randint(0, len(self) - self.batch_size)
            batch_index = random_start + torch.range(0, self.batch_size - 1)
            index[i * self.batch_size : (i + 1) * self.batch_size] = batch_index
        # deal with tail
        if tail:
            random_start = random.randint(0, len(self) - self.batch_size)
            tail_index = random_start + torch.range(0, tail - 1)
            index[(i + 1) * self.batch_size :] = tail_index

        return iter(index)

    def __len__(self):
        return self.num_samples


class alignCollate(object):
    def __init__(self, imgH=32, imgW=100, keep_ratio=False, min_ratio=1):
        self.imgH = imgH
        self.imgW = imgW
        self.keep_ratio = keep_ratio
        self.min_ratio = min_ratio

    def __call__(self, batch):
        images, labels = zip(*batch)

        imgH = self.imgH
        imgW = self.imgW
        if self.keep_ratio:
            ratios = []
            for image in images:
                w, h = image.size
                ratios.append(w / float(h))
            max_ratio = ratios[-1]
            imgW = int(np.floor(max_ratio * imgH))
            imgW = max(imgH * self.min_ratio, imgW)  # assure imgH >= imgW

        transform = resizeNormalize((imgW, imgH))
        images = [transform(image) for image in images]
        images = torch.cat([t.unsqueeze(0) for t in images], 0)

        return images, labels




import sys
import os
import torch
sys.path.append('pytorch-mobilenet-v3-master')
from mobilenetv3 import mobilenetv3

if __name__ == "__main__":
    prj_path = sys.argv[1]

    pth_file = os.path.join(prj_path,"data/models/mobilenetv3_small_67.4.pth.tar")
    pytorch_net = mobilenetv3(mode="small")
    pytorch_net.load_state_dict(torch.load(pth_file, map_location='cpu'))
    pytorch_net.eval()
    # jit.trace.save
    INPUT_SIZE = (224, 224) # h, w
    TRACED_PT = os.path.join(prj_path,"data/models/mobilenet-v3_small.torchscript.pt")
    traced_model = torch.jit.trace(pytorch_net, torch.rand(32, 3, INPUT_SIZE[0], INPUT_SIZE[1]))
    torch.jit.save(traced_model, TRACED_PT)


import os
import cv2
import numpy as np
import magicmind.python.runtime as mm

IJB_DATASETS_PATH = os.environ.get("IJB_DATASETS_PATH")

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, img_dir: str):
        super().__init__()
        with open(img_dir, 'r') as f:
            image_paths = f.readlines()
        self.images = []
        for image_path in image_paths:
            image = cv2.imread(str(IJB_DATASETS_PATH) + '/' + image_path.strip())
            assert image is not None, 'image [' + image_path.strip() + '] not exists!'
            self.images.append(image)
        nimages = len(self.images)
        assert nimages != 0, 'no images in calibrate list[' + img_dir + ']!'

        self.shape_ = shape
        if nimages < self.shape_.GetDimValue(0):
            for i in range(self.shape_.GetDimValue(0) - nimages):
                self.images.append(self.images[0])

        self.cur_sample_ = None
        self.cur_image_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_

    def preprocess_image(self):
        if self.cur_image_index_ == len(self.images):
            return None
        h,w = self.dst_shape_
        image = self.images[self.cur_image_index_]
        image = cv2.resize(image, (w, h))
        image = image.astype('float32')
        image -= 127.5
        image /= 127.5
        image = image.astype('float32')
        image = np.transpose(image, (2, 0, 1)) # HWC >>> CHW
        self.cur_image_index_ = self.cur_image_index_ + 1
        return image

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        preprocessed_images = []
        for i in range(batch_size):
            image = self.preprocess_image()
            if image is None:
                # no more data
                return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
            preprocessed_images.append(image)
        self.cur_sample_ = np.array(preprocessed_images)
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_image_index_ = 0
        return mm.Status.OK()



import os
import time 
import torch 
import numpy as np 
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
import argparse
import sys
import glob
sys.path.append('../')
from infer_python.preprocess import preprocess
import cv2

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.png')
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            img, _ = preprocess(img)
            imgs.append(img[np.newaxis,:])
        # batch and normalize
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()


def generate_config(args):
    config = mm.BuilderConfig()
    # 指定硬件平台
    assert config.parse_from_string('{"archs":[{"mtp_372": [2,6,8]}]}').ok()
    # 优化项
    assert config.parse_from_string('{"opt_config":{"type64to32_conversion": true}}').ok()
    assert config.parse_from_string('{"opt_config":{"conv_scale_fold": true}}').ok()
    assert config.parse_from_string('{"computation_preference": "fast"}').ok
    assert config.parse_from_string('{"debug_config":{"fusion_enable":false}}').ok()
    # 模型输入输出规模可变功能
    assert config.parse_from_string('{      \
        "graph_shape_mutable": true,        \
        "dim_range": {                      \
        "0": {                              \
            "min": [1, 3, 1024, 2048],      \
            "max": [16,3, 1024, 2048]       \
        }                                   \
    }}').ok()
    # 精度模式
    assert config.parse_from_string('{"precision_config":{"precision_mode":"%s"}}' % args.precision).ok()
    return config

def calibrate(args, network : mm.Network, config : mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    calib_data = CalibData(shape = mm.Dims([args.batch_size, 3, args.input_height, args.input_width]), 
                           max_samples = args.batch_size, 
                           img_dir = args.image_dir)
    calibrator = mm.Calibrator([calib_data])
    assert calibrator is not None
    # 设置量化统计算法，支持线性统计算法（LINEAR_ALGORITHM）及加强的最小化量化噪声算法（EQM_ALGORITHM）。
    assert calibrator.set_quantization_algorithm(mm.QuantizationAlgorithm.LINEAR_ALGORITHM).ok()
    # 打开设备
    with mm.System() as mm_sys:
        dev_count = mm_sys.device_count()
        print("Device count: ", dev_count)
        if args.device_id >= dev_count:
            print("Invalid device set!")
            abort()
        # 打开MLU设备
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        print("Working on MLU:", args.device_id)
    # 进行量化
    assert calibrator.calibrate(network, config).ok()


def main():
    args = argparse.ArgumentParser()
    args.add_argument('--onnx_model', dest = 'onnx_model',
            required = True, type = str, help = 'onnx model path')
    args.add_argument('--output_model', dest = 'output_model', default = '',
            type = str, help = 'output model path')
    args.add_argument('--precision', dest = 'precision', default = 'force_float32',
            type = str, help = 'precision, only qint8_mixed_float16 qint8_mixed_float32 force_float16 force_float32 qint16_mixed_float32 are supported')
    args.add_argument('--image_dir', dest = 'image_dir', default = 'image_dir',
            type = str, help = 'image list file path, file contains input image paths for calibration')
    args.add_argument('--batch_size', dest = 'batch_size', default = 1,
            type = int, help = 'batch_size')
    args.add_argument('--input_width', dest = 'input_width', default = 2048,
            type = int, help = 'model input width')
    args.add_argument('--input_height', dest = 'input_height', default = 1024,
            type = int, help = 'model input height')
    args.add_argument('--device_id', dest = 'device_id', default = 0,
            type = int, help = 'mlu device id, used for calibration')
    args = args.parse_args()
    print(args)
    supported_precision = ['qint8_mixed_float16', 'qint8_mixed_float32', 'qint16_mixed_float16', 'qint16_mixed_float32', 'force_float16', 'force_float32']
    if args.precision not in supported_precision:
        print('precision [' + args.precision + ']', 'not supported')
        exit()

    # 生成mm模型
    parser = Parser(mm.ModelKind.kOnnx)
    network = mm.Network()
    assert parser.parse(network, args.onnx_model).ok()
    build_config = generate_config(args)

    if args.precision.find('qint') != -1:
        print('do calibrate...')
        calibrate(args, network, build_config)
    print('build model...')

    builder = mm.Builder()
    model = builder.build_model("ocrnet", network, build_config)
    if '' == args.output_model:
        args.output_model = 'ocrnet_{}.mm'.format(args.precision)
    assert model.serialize_to_file(args.output_model).ok()
    print("Generate model done, model save to %s" % args.output_model) 

if __name__=='__main__':
    main()



import numpy as np
import magicmind.python.runtime as mm
import cv2
import os
import glob
import math
from PIL import Image

from logger import Logger

log = Logger()


def coco_dataset(
    file_list_txt="coco_file_list_5000.txt",
    image_dir="../../../../datasets/coco/val2017",
    count=-1,
):
    with open(file_list_txt, "r") as f:
        lines = f.readlines()
    #log.info("%d pictures will be read." % len(lines))
    current_count = 0
    for line in lines:
        image_name = line.strip()
        image_path = os.path.join(image_dir, image_name)
        img = cv2.imread(image_path)
        yield img, image_path
        current_count += 1
        if current_count >= count and count != -1:
            break

def image_preporcess(image, target_size, gt_boxes=None):

    image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB).astype(np.float32)
    ih, iw    = target_size
    h,  w, _  = image.shape
    scale = min(iw/w, ih/h)
    nw, nh  = int(scale * w), int(scale * h)
    image_resized = cv2.resize(image, (nw, nh))
    image_paded = np.full(shape=[ih, iw, 3], fill_value=128.0)
    dw, dh = (iw - nw) // 2, (ih-nh) // 2
    image_paded[dh:nh+dh, dw:nw+dw, :] = image_resized
    image_paded = image_paded / 255.
    if gt_boxes is None:
        return image_paded
    else:
        gt_boxes[:, [0, 2]] = gt_boxes[:, [0, 2]] * scale + dw
        gt_boxes[:, [1, 3]] = gt_boxes[:, [1, 3]] * scale + dh
        return image_paded, gt_boxes

def preprocess_image(img, input_size) -> np.ndarray:

    original_image = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    image_data = image_preporcess(np.copy(original_image), [input_size, input_size])
    image_data = image_data.astype(np.float32)    
    return image_data

class CalibData(mm.CalibDataInterface):
    def __init__(self, shape: mm.Dims, max_samples: int, img_dir: str):
        super().__init__()
        assert os.path.isdir(img_dir)
        self.data_paths_ = glob.glob(img_dir + '/*.jpg')
        self.shape_ = shape
        self.max_samples_ = min(max_samples, len(self.data_paths_))
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        self.dst_shape_ = (self.shape_.GetDimValue(2), self.shape_.GetDimValue(3))

    def get_shape(self):
        return self.shape_

    def get_data_type(self):
        return mm.DataType.FLOAT32

    def get_sample(self):
        return self.cur_sample_
    
    def preprocess_images(self, data_begin: int, data_end: int) -> np.ndarray:
        imgs = []
        for i in range(data_begin, data_end):
            img = cv2.imread(self.data_paths_[i])
            img = preprocess_image(img, self.dst_shape_[0])
            imgs.append(img[np.newaxis,:])
        # batch and normalize
        return np.ascontiguousarray(np.concatenate(tuple(imgs), axis=0))

    def next(self):
        batch_size = self.shape_.GetDimValue(0)
        data_begin = self.cur_data_index_
        data_end = data_begin + batch_size
        if data_end > self.max_samples_:
            return mm.Status(mm.Code.OUT_OF_RANGE, "Data end reached")
        self.cur_sample_ = self.preprocess_images(data_begin, data_end)
        self.cur_data_index_ = data_end
        return mm.Status.OK()

    def reset(self):
        self.cur_sample_ = None
        self.cur_data_index_ = 0
        return mm.Status.OK()

    def new_video(self, path):
        self.frame = 0
        self.cap = cv2.VideoCapture(path)
        self.nframes = int(self.cap.get(cv2.CAP_PROP_FRAME_COUNT))

    def __len__(self):
        return self.nf  # number of files



import tensorflow.compat.v1 as tf
from tensorflow.core.framework import node_def_pb2
from tensorflow.core.framework import attr_value_pb2
from tensorflow.python.framework import tensor_util

input_model = "../data/models/roformer.pb"
output_model_path = "../data/models/"
output_model_name = "roformer_const.pb"

def create_node(op, name, inputs):
  new_node = node_def_pb2.NodeDef()
  new_node.op = op
  new_node.name = name
  for input_name in inputs:
    new_node.input.extend([input_name])
  return new_node

def create_constant_node(name, value, dtype, shape=None):
  node = create_node("Const", name, [])
  set_attr_dtype(node, "dtype", dtype)
  set_attr_tensor(node, "value", value, dtype, shape)
  return node

def set_attr_dtype(node, key, value):
  try:
    node.attr[key].CopyFrom(
        attr_value_pb2.AttrValue(type=value.as_datatype_enum))
  except KeyError:
    pass

def set_attr_tensor(node, key, value, dtype, shape=None):
  try:
    node.attr[key].CopyFrom(
        attr_value_pb2.AttrValue(tensor=tensor_util.make_tensor_proto(
            value, dtype=dtype, shape=shape)))
  except KeyError:
    pass

with tf.Graph().as_default() as graph:
  in_graph_def = tf.GraphDef()
  out_graph_def = tf.GraphDef()
  with tf.gfile.GFile(input_model, 'rb') as fid:
    serialized_graph = fid.read()
    in_graph_def.ParseFromString(serialized_graph)
    tf.import_graph_def(in_graph_def, name='')
    for node in in_graph_def.node:
      if node.name in ["keras_learning_phase"]:
        new_const = create_constant_node(node.name, False, tf.bool, shape=[])
        out_graph_def.node.extend([new_const])
      else:
        out_graph_def.node.extend([node])
    if in_graph_def.library:
      out_graph_def.library.CopyFrom(in_graph_def.library)     
    
    tf.train.write_graph(out_graph_def, output_model_path , output_model_name, False)
    
print("stage 1 finish ")



import paddle
try:
    import paddle._legacy_C_ops as C_ops
except:
    import paddle._C_ops as C_ops

from paddle import in_dynamic_mode
from paddle.common_ops_import import Variable, LayerHelper, check_variable_and_dtype, check_type, check_dtype

@paddle.jit.not_to_static
def multiclass_nms(bboxes,
                   scores,
                   score_threshold,
                   nms_top_k,
                   keep_top_k,
                   nms_threshold=0.3,
                   normalized=True,
                   nms_eta=1.,
                   background_label=-1,
                   return_index=False,
                   return_rois_num=True,
                   rois_num=None,
                   name=None):
    """
    This operator is to do multi-class non maximum suppression (NMS) on
    boxes and scores.
    In the NMS step, this operator greedily selects a subset of detection bounding
    boxes that have high scores larger than score_threshold, if providing this
    threshold, then selects the largest nms_top_k confidences scores if nms_top_k
    is larger than -1. Then this operator pruns away boxes that have high IOU
    (intersection over union) overlap with already selected boxes by adaptive
    threshold NMS based on parameters of nms_threshold and nms_eta.
    Aftern NMS step, at most keep_top_k number of total bboxes are to be kept
    per image if keep_top_k is larger than -1.
    Args:
        bboxes (Tensor): Two types of bboxes are supported:
                           1. (Tensor) A 3-D Tensor with shape
                           [N, M, 4 or 8 16 24 32] represents the
                           predicted locations of M bounding bboxes,
                           N is the batch size. Each bounding box has four
                           coordinate values and the layout is
                           [xmin, ymin, xmax, ymax], when box size equals to 4.
                           2. (LoDTensor) A 3-D Tensor with shape [M, C, 4]
                           M is the number of bounding boxes, C is the
                           class number
        scores (Tensor): Two types of scores are supported:
                           1. (Tensor) A 3-D Tensor with shape [N, C, M]
                           represents the predicted confidence predictions.
                           N is the batch size, C is the class number, M is
                           number of bounding boxes. For each category there
                           are total M scores which corresponding M bounding
                           boxes. Please note, M is equal to the 2nd dimension
                           of BBoxes.
                           2. (LoDTensor) A 2-D LoDTensor with shape [M, C].
                           M is the number of bbox, C is the class number.
                           In this case, input BBoxes should be the second
                           case with shape [M, C, 4].
        background_label (int): The index of background label, the background
                                label will be ignored. If set to -1, then all
                                categories will be considered. Default: 0
        score_threshold (float): Threshold to filter out bounding boxes with
                                 low confidence score. If not provided,
                                 consider all boxes.
        nms_top_k (int): Maximum number of detections to be kept according to
                         the confidences after the filtering detections based
                         on score_threshold.
        nms_threshold (float): The threshold to be used in NMS. Default: 0.3
        nms_eta (float): The threshold to be used in NMS. Default: 1.0
        keep_top_k (int): Number of total bboxes to be kept per image after NMS
                          step. -1 means keeping all bboxes after NMS step.
        normalized (bool): Whether detections are normalized. Default: True
        return_index(bool): Whether return selected index. Default: False
        rois_num(Tensor): 1-D Tensor contains the number of RoIs in each image.
            The shape is [B] and data type is int32. B is the number of images.
            If it is not None then return a list of 1-D Tensor. Each element
            is the output RoIs' number of each image on the corresponding level
            and the shape is [B]. None by default.
        name(str): Name of the multiclass nms op. Default: None.
    Returns:
        A tuple with two Variables: (Out, Index) if return_index is True,
        otherwise, a tuple with one Variable(Out) is returned.
        Out: A 2-D LoDTensor with shape [No, 6] represents the detections.
        Each row has 6 values: [label, confidence, xmin, ymin, xmax, ymax]
        or A 2-D LoDTensor with shape [No, 10] represents the detections.
        Each row has 10 values: [label, confidence, x1, y1, x2, y2, x3, y3,
        x4, y4]. No is the total number of detections.
        If all images have not detected results, all elements in LoD will be
        0, and output tensor is empty (None).
        Index: Only return when return_index is True. A 2-D LoDTensor with
        shape [No, 1] represents the selected index which type is Integer.
        The index is the absolute value cross batches. No is the same number
        as Out. If the index is used to gather other attribute such as age,
        one needs to reshape the input(N, M, 1) to (N * M, 1) as first, where
        N is the batch size and M is the number of boxes.
    Examples:
        .. code-block:: python

            import paddle
            from ppdet.modeling import ops
            boxes = paddle.static.data(name='bboxes', shape=[81, 4],
                                      dtype='float32', lod_level=1)
            scores = paddle.static.data(name='scores', shape=[81],
                                      dtype='float32', lod_level=1)
            out, index = ops.multiclass_nms(bboxes=boxes,
                                            scores=scores,
                                            background_label=0,
                                            score_threshold=0.5,
                                            nms_top_k=400,
                                            nms_threshold=0.3,
                                            keep_top_k=200,
                                            normalized=False,
                                            return_index=True)
    """
    helper = LayerHelper('multiclass_nms3', **locals())

    if in_dynamic_mode():
        attrs = ('background_label', background_label, 'score_threshold',
                 score_threshold, 'nms_top_k', nms_top_k, 'nms_threshold',
                 nms_threshold, 'keep_top_k', keep_top_k, 'nms_eta', nms_eta,
                 'normalized', normalized)
        output, index, nms_rois_num = C_ops.multiclass_nms3(bboxes, scores,
                                                            rois_num, *attrs)
        if not return_index:
            index = None
        return output, nms_rois_num, index

    else:
        output = helper.create_variable_for_type_inference(dtype=bboxes.dtype)
        index = helper.create_variable_for_type_inference(dtype='int32')

        inputs = {'BBoxes': bboxes, 'Scores': scores}
        outputs = {'Out': output, 'Index': index}

        if rois_num is not None:
            inputs['RoisNum'] = rois_num

        if return_rois_num:
            nms_rois_num = helper.create_variable_for_type_inference(
                dtype='int32')
            outputs['NmsRoisNum'] = nms_rois_num

        helper.append_op(
            type="multiclass_nms3",
            inputs=inputs,
            attrs={
                'background_label': background_label,
                'score_threshold': score_threshold,
                'nms_top_k': nms_top_k,
                'nms_threshold': nms_threshold,
                'keep_top_k': keep_top_k,
                'nms_eta': nms_eta,
                'normalized': normalized
            },
            outputs=outputs)
        output.stop_gradient = True
        index.stop_gradient = True
        if not return_index:
            index = None
        if not return_rois_num:
            nms_rois_num = None

        return output, nms_rois_num, index



import magicmind.python.runtime as mm
from batchgenerators.utilities.file_and_folder_operations import *
from batchgenerators.augmentations.utils import pad_nd_image
import argparse
import numpy as np
import torch
import os
import cv2
from mmseg.core.evaluation import get_palette
from preprocess import preprocess
from mmseg.datasets import build_dataloader, build_dataset
import mmcv
import warnings

parser = argparse.ArgumentParser()
parser.add_argument('--device_id', dest = 'device_id', default = 0,
            type = int, help = 'mlu device id, used for calibration')
parser.add_argument("--magicmind_model", type=str, default="magicmind_model", help="magicmind_model")
parser.add_argument("--config", type=str, default="../export_model/mmsegmentation/configs/ocrnet/ocrnet_hr18_512x1024_160k_cityscapes.py", 
                        help="model config")
parser.add_argument("--data_root", type=str, default="../data/cityscapes", help="dataset dir")
parser.add_argument("--json_file", type=str, default="result.json", help="save eval result")
CLASS=('road', 'sidewalk', 'building', 'wall', 
        'fence', 'pole', 'traffic light', 'traffic sign', 
        'vegetation', 'terrain', 'sky', 'person', 'rider', 
        'car', 'truck', 'bus', 'train', 'motorcycle', 'bicycle')
PALETTE=[[128, 64, 128], [244, 35, 232], [70, 70, 70], [102, 102, 156], 
        [190, 153, 153], [153, 153, 153], [250, 170, 30], [220, 220, 0], 
        [107, 142, 35], [152, 251, 152], [70, 130, 180], [220, 20, 60], 
        [255, 0, 0], [0, 0, 142], [0, 0, 70], [0, 60, 100], [0, 80, 100], 
        [0, 0, 230], [119, 11, 32]]

def show_result(img,
                result,
                palette=None,
                opacity=0.5):
        """Draw `result` over `img`.

        Args:
            img (str or Tensor): The image to be displayed.
            result (Tensor): The semantic segmentation results to draw over
                `img`.
            palette (list[list[int]]] | np.ndarray | None): The palette of
                segmentation map. If None is given, random palette will be
                generated. Default: None
            win_name (str): The window name.
            wait_time (int): Value of waitKey param.
                Default: 0.
            show (bool): Whether to show the image.
                Default: False.
            out_file (str or None): The filename to write the image.
                Default: None.
            opacity(float): Opacity of painted segmentation map.
                Default 0.5.
                Must be in (0, 1] range.
        Returns:
            img (Tensor): Only if not `show` or `out_file`
        """
        img = img.copy()
        seg = result
        palette = np.array(palette)
        assert palette.shape[0] == 19
        assert palette.shape[1] == 3
        assert len(palette.shape) == 2
        assert 0 < opacity <= 1.0
        color_seg = np.zeros((seg.shape[0], seg.shape[1], 3), dtype=np.uint8)
        for label, color in enumerate(palette):
            color_seg[seg == label, :] = color
        # convert to BGR
        color_seg = color_seg[..., ::-1]

        img = img * (1 - opacity) + color_seg * opacity
        img = img.astype(np.uint8)
        # if out_file specified, do not show image in window
        return img

def single_mlu_test(context,
                    queue,
                    data_loader,
                    show=False,
                    out_dir=None,
                    efficient_test=False,
                    opacity=0.5,
                    pre_eval=False,
                    format_only=False,
                    format_args={}):
    """Test with single GPU by progressive mode.

    Args:
        model (mm.Model): Model to be tested.
        data_loader (utils.data.Dataloader): Pytorch data loader.
        show (bool): Whether show results during inference. Default: False.
        out_dir (str, optional): If specified, the results will be dumped into
            the directory to save output results.
        efficient_test (bool): Whether save the results as local numpy files to
            save CPU memory during evaluation. Mutually exclusive with
            pre_eval and format_results. Default: False.
        opacity(float): Opacity of painted segmentation map.
            Default 0.5.
            Must be in (0, 1] range.
        pre_eval (bool): Use dataset.pre_eval() function to generate
            pre_results for metric evaluation. Mutually exclusive with
            efficient_test and format_results. Default: False.
        format_only (bool): Only format result for results commit.
            Mutually exclusive with pre_eval and efficient_test.
            Default: False.
        format_args (dict): The args for format_results. Default: {}.
    Returns:
        list: list of evaluation pre-results or list of save file names.
    """
    if efficient_test:
        warnings.warn(
            'DeprecationWarning: ``efficient_test`` will be deprecated, the '
            'evaluation is CPU memory friendly with pre_eval=True')
        mmcv.mkdir_or_exist('.efficient_test')
    # when none of them is set true, return segmentation results as
    # a list of np.array.
    assert [efficient_test, pre_eval, format_only].count(True) <= 1, \
        '``efficient_test``, ``pre_eval`` and ``format_only`` are mutually ' \
        'exclusive, only one of them could be true .'

    inputs = context.create_inputs()
    dataset = data_loader.dataset
    prog_bar = mmcv.ProgressBar(len(dataset))
    # The pipeline about how the data_loader retrieval samples from dataset:
    # sampler -> batch_sampler -> indices
    # The indices are passed to dataset_fetcher to get data from dataset.
    # data_fetcher -> collate_fn(dataset[index]) -> data_sample
    # we use batch_sampler to get correct data idx
    loader_indices = data_loader.batch_sampler
    results = []
    for batch_indices, data in zip(loader_indices, data_loader):
        outputs = []
        img_tensor = data['img'][0]
        img_metas = data['img_metas'][0].data[0]
        ori_h, ori_w = img_metas[0]['ori_shape'][:-1]
        with torch.no_grad():
            inputs[0].from_numpy(img_tensor.numpy())
            context.enqueue(inputs, outputs, queue)
            queue.sync()
            result = outputs[0].asnumpy()
            result = result.reshape(result.shape[2], result.shape[3], 1).astype(np.uint8)
            preds = cv2.resize(result, (ori_w, ori_h), dst=None, interpolation=cv2.INTER_NEAREST)
            # preds = np.expand_dims(preds, axis=0)
            # print(preds.shape)

        if show or out_dir:            
            imgs = tensor2imgs(img_tensor, **img_metas[0]['img_norm_cfg'])
            assert len(imgs) == len(img_metas)

            for img, img_meta in zip(imgs, img_metas):
                h, w, _ = img_meta['img_shape']
                img_show = img[:h, :w, :]

                img_show = mmcv.imresize(img_show, (ori_w, ori_h))

                if out_dir:
                    out_file = osp.join(out_dir, img_meta['ori_filename'])
                else:
                    out_file = None

                model.module.show_result(
                    img_show,
                    result,
                    palette=dataset.PALETTE,
                    show=show,
                    out_file=out_file,
                    opacity=opacity)

        if efficient_test:
            result = [np2tmp(_, tmpdir='.efficient_test') for _ in result]

        if format_only:
            result = dataset.format_results(
                result, indices=batch_indices, **format_args)
        if pre_eval:
            # TODO: adapt samples_per_gpu > 1.
            # only samples_per_gpu=1 valid now
            result = dataset.pre_eval(preds, indices=batch_indices)
            results.extend(result)
        else:
            results.extend(result)

        batch_size = len(result)
        for _ in range(batch_size):
            prog_bar.update()

    return results

if __name__ == "__main__":
    args = parser.parse_args()
    if not os.path.exists(args.magicmind_model):
        print("please generate magicmind {} first!!!".format(args.magicmind_model))
        exit()

    with mm.System():
        model = mm.Model()
        model.deserialize_from_file(args.magicmind_model)
        dev = mm.Device()
        dev.id = args.device_id
        assert dev.active().ok()
        econfig = mm.Model.EngineConfig()
        econfig.device_type = "MLU"
        engine = model.create_i_engine(econfig)
        assert engine != None, "Failed to create engine"
        context = engine.create_i_context()
        queue = dev.create_queue()
        assert queue != None

        with torch.no_grad():
            cfg = mmcv.Config.fromfile(args.config)
            cfg.data.test['data_root']=args.data_root
            dataset = build_dataset(cfg.data.test)
            cfg.data.test.test_mode = True
            test_loader_cfg = {'num_gpus': 1, 'dist': False, 'shuffle': False, 'samples_per_gpu': 1, 'workers_per_gpu': 2}
            eval_kwargs={'imgfile_prefix': '.format_cityscapes'}
            data_loader = build_dataloader(dataset, **test_loader_cfg)
            results = single_mlu_test(context,
                                    queue,
                                    data_loader,
                                    False,
                                    None,
                                    False,
                                    0.5,
                                    pre_eval=True,
                                    format_only=False,
                                    format_args=eval_kwargs)
            eval_kwargs.update(metric='mIoU')
            metric = dataset.evaluate(results, **eval_kwargs)
            metric_dict = dict(config=args.config, metric=metric)
            mmcv.dump(metric_dict, args.json_file, indent=4)



import magicmind.python.runtime as mm
from calibrator import CalibData
import time
  
# import common components
from framework_parser import OnnxParser
from common_calibrate import common_calibrate
from build_param import get_argparser
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)
from logger import Logger
# 实例化python logger类
log = Logger()

def calibrate(args, network : mm.Network, config : mm.BuilderConfig):
    custom_max_samples = 10
    max_samples = max(custom_max_samples, args.input_dims[0][0])
    calib_data = CalibData(args, mm.Dims([args.input_dims[0][0], args.input_dims[0][1], args.input_dims[0][2], args.input_dims[0][3]]), 
                            max_samples, args.image_dir)
    common_calibrate(args, network, config, calib_data)

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()

    return network

def get_args():
    # get common argparser,here is pytorch_parser
    arg_parser = get_argparser()

    # add custom args belonging to the current net.
    arg_parser.add_argument("--task", type=str, default="det", help="det, rec or cls")
    
    # params for text detection    
    arg_parser.add_argument("--image_dir", type=str, default="../doc/imgs/11.jpg", help='det_image_dir')
    arg_parser.add_argument("--det_batch_num", type=int, default=1, help='det_batch_num')
    arg_parser.add_argument("--det_limit_side_len", type=float, default=1280, help='det_limit_side_len')
    arg_parser.add_argument("--det_limit_type", type=str, default='max', help='det_limit_type')

    # params for text recognizer    
    # arg_parser.add_argument("--rec_image_shape", type=str, default="3, 48, 320", help='rec_image_shape')
    arg_parser.add_argument("--rec_batch_num", type=int, default=6, help='rec_batch_num,default is 6')
    arg_parser.add_argument("--limited_max_width", type=int, default=1280, help='limited_max_width')
    arg_parser.add_argument("--limited_min_width", type=int, default=16, help='limited_min_width')

    #params for text classifiler
    arg_parser.add_argument("--cls_model_path", type=str,default="", help='cls_model_path,default is:../data/models/ch_ppocr_mobile_v2.0_cls_infer.onnx')
    arg_parser.add_argument("--cls_batch_num", type=int,default=6,  help='cls_batch_num,default is 6')
    # arg_parser.add_argument("--cls_image_shape",type=str, default="3, 48, 192")
    args = arg_parser.parse_args()

    if args.task not in ['det', 'rec', 'cls']:
        log.err("Please input  det, rec or cls!")


    if args.precision == 'qint8_mixed_float16' and args.task == 'rec':
        log.err("OCR REC model not support qint8_mixed_float16!!! set 'force_float32' or 'force_float16' !")

    return args

def main():
    # get net args
    begin_time = time.time()

    args = get_args()
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)    

    if args.precision.find('qint') != -1:
        log.info('Do calibrate...')
        calibrate(args, network, build_config)
    log.info('build model...')
    # 生成模型并导出
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == '__main__':
    main()



import argparse
import magicmind.python.runtime as mm
from magicmind.python.runtime.parser import Parser
from magicmind.python.runtime import DataType
from calibrator import CalibData
import time
import sys
import numpy as np

# import common components
from framework_parser import OnnxParser
from common_calibrate import common_calibrate
from build_param import get_argparser 
from model_process import (
    extract_params,
    config_network,
    get_builder_config,
    build_and_serialize,
)

import torch
import mmcv
from mmcv import Config, DictAction
from mmpose.datasets import build_dataloader, build_dataset

from logger import Logger

log = Logger()

def get_calibrate_data(_config,data_num):
    cfg = Config.fromfile(_config)
    
    # build the dataloader
    dataset = build_dataset(cfg.data.test, dict(test_mode=True))
    # step 1: give default values and override (if exist) from cfg.data
    loader_cfg = {
        **dict(seed=cfg.get('seed'), drop_last=False, dist=False),
        **({} if torch.__version__ != 'parrots' else dict(
               prefetch_num=2,
               pin_memory=False,
           )),
        **dict((k, cfg.data[k]) for k in [
                   'seed',
                   'prefetch_num',
                   'pin_memory',
                   'persistent_workers',
               ] if k in cfg.data)
    }
    # step2: cfg.data.test_dataloader has higher priority
    test_loader_cfg = {
        **loader_cfg,
        **dict(shuffle=False, drop_last=False),
        **dict(workers_per_gpu=1),
        **dict(samples_per_gpu=1),
        **cfg.data.get('test_dataloader', {})
    }
    # print()
    data_loader = build_dataloader(dataset, **test_loader_cfg)
    
    iter_idx = 0
    calib_data = []
    calib_finshed = False
    for data in data_loader:
        while not calib_finshed:
            img_tensor = data['img']
            assert img_tensor.size(0) == len(data['img_metas'].data[0])
            img_metas = data['img_metas'].data[0][0]
            aug_data = img_metas['aug_data']
            test_scale_factor = img_metas['test_scale_factor']
            for idx, s in enumerate(sorted(test_scale_factor, reverse=True)):
                if iter_idx < data_num:
                    image_resized = aug_data[idx].to(img_tensor.device)
                    np_data = image_resized[0].numpy()
                    calib_data.append(np_data)     
                else:
                    calib_finshed = True
                    break
                iter_idx += 1
    return calib_data

def get_network(args):
    parser = OnnxParser(args)
    network = parser.parse()
    return network

def calibrate(args, network: mm.Network, config: mm.BuilderConfig):
    # 创建量化工具并设置量化统计算法
    input_dims = mm.Dims(args.input_dims[0])
    sample_data = get_calibrate_data(args.config,args.batch_size[0])
    calib_data = CalibData(shape=input_dims, max_samples=args.batch_size[0], sample_data = sample_data)
    common_calibrate(args, network, config, calib_data)

def get_args():
    # get common argparser,here is pytorch_parser
    arg_parser = get_argparser()

    arg_parser.add_argument(
        "--config",
        dest="config",
        type=str,
        default="",
        help="config",
    )
    return arg_parser.parse_args()

def main():
    begin_time = time.time()
    # get net args
    args = get_args()
    network = get_network(args)
    # configure network, such as input_dim, batch_size ...
    config_args = extract_params("MODEL_CONFIG", args)
    config_network(network, config_args)
    # create build configuration
    builder_args = extract_params("MODEL_BUILDER", args)
    build_config = get_builder_config(builder_args)

    if args.precision.find("qint") != -1:
        log.info("Do calibrate...")
        calibrate(args, network, build_config)
    log.info("Build model...")
    # 生成模型并导出
    model_name = "network"
    # build_and_serialize_params = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize_args = extract_params("MODEL_BUILD_AND_SERIALIZE", args)
    build_and_serialize(network, build_config, build_and_serialize_args)

    end_time = time.time()
    log.info("gen_model time cost:{:.3f}s".format(end_time - begin_time))

if __name__ == "__main__":
    main()



import os
import cv2
import numpy as np
from PIL import Image
from logger import Logger
from torchvision import transforms

log = Logger()


def imagenet_dataset(val_txt, image_file_path, count=-1):
    with open(val_txt, "r") as f:
        lines = f.readlines()
    log.info("%d pictures will be read." % len(lines))
    current_count = 1
    for line in lines:
        image_name, label = line.split(" ")
        image_path = os.path.join(image_file_path, image_name)
        img = cv2.imread(image_path)
        yield img, label.strip()
        current_count += 1
        if current_count > count and count != -1:
            break


def preprocess(input_image, transpose):
    resize_h, resize_w = (256, 256)
    crop_h, crop_w = (224, 224)
    # resize
    scale = 1.0 * resize_h / min(input_image.shape[0], input_image.shape[1])
    resized = cv2.resize(
        input_image,
        (round(scale * input_image.shape[1]), round(scale * input_image.shape[0])),
    )
    # # center crop
    x = resized.shape[1] / 2 - crop_w / 2
    y = resized.shape[0] / 2 - crop_h / 2
    crop_img = resized[int(y) : int(y + crop_h), int(x) : int(x + crop_w)]
    if transpose:
        input_numpy = np.transpose(crop_img, (1, 2, 0))
    else:
        input_numpy = crop_img
    return input_numpy


def load_name(imagenet_label_path):
    name_map = {}
    with open(imagenet_label_path, "r") as f:
        lines = f.readlines()
    for line in lines:
        idx = line.split(" ")[0]
        name = " ".join(line.split(" ")[1:])
        name_map[int(idx)] = name.strip()
    return name_map
